#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess
import stat
import tarfile
import time
import hashlib

import anchore.anchore_utils

analyzer_name = "file_checksums"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
imgid = config['imgid_full']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

#if not os.path.exists(outputdir):
#    os.makedirs(outputdir)

domd5 = True
outfiles_md5 = {}
outfiles_sha256 = {}

try:
    timer = time.time()
    (tmp, allfiles) = anchore.anchore_utils.get_files_from_path(unpackdir + "/rootfs")
    for name in allfiles.keys():
        name = re.sub("^\.", "", name)
        thefile = '/'.join([unpackdir, "rootfs", name])

        csum = "DIRECTORY_OR_OTHER"
        if os.path.isfile(thefile) and not os.path.islink(thefile):
            if domd5:
                csum = "DIRECTORY_OR_OTHER"
                try:
                    with open(thefile, 'r') as FH:
                        csum = hashlib.md5(FH.read()).hexdigest()
                except:
                    pass
                outfiles_md5[name] = csum

            try:
                with open(thefile, 'r') as FH:
                    csum = hashlib.sha256(FH.read()).hexdigest()
            except:
                pass

            outfiles_sha256[name] = csum

        else:
            outfiles_md5[name] = "DIRECTORY_OR_OTHER"
            outfiles_sha256[name] = "DIRECTORY_OR_OTHER"

except Exception as err:
    import traceback
    traceback.print_exc()
    print "ERROR: " + str(err)
    raise err

if outfiles_md5:
    ofile = os.path.join(outputdir, 'files.md5sums')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, outfiles_md5)

if outfiles_sha256:
    ofile = os.path.join(outputdir, 'files.sha256sums')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, outfiles_sha256)


sys.exit(0)



#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <all|summary> ...\nhelp: use 'all' to show distros by version, use 'summarize' to show distro count with any version")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

onlysummarize = False
for name in config['params']:
    if name == 'summarize':
        onlysummarize = True
        break

outlist = list()
warns = list()
outlist.append(["Distro", "Distro_Version", "*Image_Count", "Image_Ids"])

result = {}

allimages = {}
for imageId in config['images']:
    try:
        image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)

        distro = image.get_distro()
        distro_vers = image.get_distro_vers()
        distro_str = distro + ":" + distro_vers

        if distro not in result:
            result[distro] = {}

        if distro_vers not in result[distro]:
            result[distro][distro_vers] = {'count':0, 'imageIds':list()}

        result[distro][distro_vers]['count'] = result[distro][distro_vers]['count'] + 1
        result[distro][distro_vers]['imageIds'].append(image.meta['imageId'])

    except Exception as err:
        warns.append(["Failed to read distro/version from image: " + image.meta['imageId']])

sums = {}
for d in result.keys():
    for v in result[d].keys():
        if not onlysummarize:
            #outlist.append([d, v, str(result[d][v]['count']), ','.join(result[d][v]['imageIds'])])
            outlist.append([d, v, str(result[d][v]['count']), ' '.join(result[d][v]['imageIds'])])

        if d not in sums:
            sums[d] = {'count':0, 'imageIds':list()}
        sums[d]['count'] = sums[d]['count'] + result[d][v]['count']
        sums[d]['imageIds'] = sums[d]['imageIds'] + result[d][v]['imageIds']

if onlysummarize:
    for k in sums.keys():
        #outlist.append([k, 'any', str(sums[k]['count']), ','.join(sums[k]['imageIds'])])
        outlist.append([k, 'any', str(sums[k]['count']), ' '.join(sums[k]['imageIds'])])

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_kvfile_fromlist(config['output_warns'], warns)

allimages.clear()
sys.exit(0)



#!/usr/bin/env python

import sys
import os
import re
import traceback

import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: all | <file> <file> ...\nhelp: produce a list of all files that have been successfully retrieved and stored during analysis of this image.  Files listed here can be used as input to the get-retrieved-files query.")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    print "Query requires input: all | <fileA> <fileB> ..."

warns = list()
outlist = list()
outlist.append(["Image_Id", "Repo_Tags", "Stored_Namespace", "Stored_Filename", "Stored_File_Size_Bytes"])

tags = "none"
if config['meta']['humanname']:
    tags = config['meta']['humanname']
imgid = config['meta']['shortId']

try:
    # handle the good case, something is found resulting in data matching the required columns
    namespaces = anchore.anchore_utils.load_files_namespaces(config['imgid'])
    for namespace in namespaces:
        stored_files = anchore.anchore_utils.load_files_metadata(config['imgid'], namespace)
        for f in stored_files.keys():
            scrubbed_name = re.sub("imageroot", "", f)
            scrubbed_size = str(stored_files[f]['size'])
            outlist.append([imgid, tags, namespace, scrubbed_name, scrubbed_size])
except Exception as err:
    # handle the case where something wrong happened
    warns.append("Unable to load stored files data - try re-analyzing image")
    import traceback
    traceback.print_exc()
    print str(err)

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

sys.exit(0)






import yaml
import os
import shutil
import filecmp
import datetime
import tarfile

from util.tools import load_and_merge
from pkg_resources import Requirement, resource_filename


class AnchoreConfiguration (object):
    """
    Backed by a yaml file
    """

    DEFAULT_ANCHORE_DATA_DIR = os.path.join(os.getenv('HOME'), '.anchore')
    DEFAULT_CONFIG_DIR = os.path.join(DEFAULT_ANCHORE_DATA_DIR, 'conf')
    DEFAULT_CONFIG_FILE = os.path.join(DEFAULT_CONFIG_DIR, 'config.yaml')
    DEFAULT_TMP_DIR = os.path.join(DEFAULT_ANCHORE_DATA_DIR, 'anchoretmp')
    EXAMPLE_CONFIG_DIR = resource_filename("anchore", "conf/")
    EXAMPLE_CONFIG_FILE = resource_filename("anchore", "conf/config.yaml")
    DEFAULT_PKG_DIR = resource_filename("anchore", "")
    DEFAULT_SCRIPTS_DIR = resource_filename("anchore", "anchore-modules")
    DEFAULT_ANON_ANCHORE_USERNAME = 'anon@ancho.re'
    DEFAULT_ANON_ANCHORE_PASSWORD = 'pbiU2RYZ2XrmYQ'
    DEFAULT_ANCHORE_CLIENT_URL = 'https://ancho.re/v1/account/users'
    DEFAULT_ANCHORE_TOKEN_URL = 'https://ancho.re/oauth/token'
    DEFAULT_ANCHORE_FEEDS_URL = 'https://ancho.re/v1/service/feeds'
    DEFAULT_ANCHORE_AUTH_CONN_TIMEOUT = 5
    DEFAULT_ANCHORE_AUTH_MAX_RETRIES = 3
    DEFAULT_ANCHORE_FEEDS_CONN_TIMEOUT = 10
    DEFAULT_ANCHORE_FEEDS_MAX_RETRIES = 3
    DEFAULT_ANCHORE_DB_DRIVER = "anchore_image_db_fs"
    DEFAULT_ANCHORE_SQUASH_DRIVER = "docker_export"

    try:
        DEFAULT_EXTRASCRIPTS_DIR = resource_filename("anchore-modules", "/")
    except:
        DEFAULT_EXTRASCRIPTS_DIR = None

    DEFAULTS = {
        'anchore_data_dir': DEFAULT_ANCHORE_DATA_DIR,
        'anchore_db_driver': DEFAULT_ANCHORE_DB_DRIVER,
        'feeds_dir': 'feeds',
        'feeds_url': DEFAULT_ANCHORE_FEEDS_URL,
        'feeds_conn_timeout': DEFAULT_ANCHORE_FEEDS_CONN_TIMEOUT,
        'feeds_max_retries': DEFAULT_ANCHORE_FEEDS_MAX_RETRIES,
        'image_data_store': 'data',
        'tmpdir': DEFAULT_TMP_DIR,
        'pkg_dir': DEFAULT_PKG_DIR,
        'scripts_dir': DEFAULT_SCRIPTS_DIR,
        'user_scripts_dir': 'user-scripts',
        'extra_scripts_dir': DEFAULT_EXTRASCRIPTS_DIR,
        'docker_conn': 'unix://var/run/docker.sock',
        'docker_conn_timeout':'120',
        'anchore_client_url':DEFAULT_ANCHORE_CLIENT_URL,
        'anchore_token_url':DEFAULT_ANCHORE_TOKEN_URL,
        'anchore_auth_conn_timeout':DEFAULT_ANCHORE_AUTH_CONN_TIMEOUT,
        'anchore_auth_max_retries':DEFAULT_ANCHORE_AUTH_MAX_RETRIES,
        'squash_driver':DEFAULT_ANCHORE_SQUASH_DRIVER
    }

    def __init__(self, cliargs=None):
        # config file handling

        # handle override of the default .anchore location
        if os.getenv('ANCHOREDATADIR'):
            self.DEFAULT_ANCHORE_DATA_DIR = os.getenv('ANCHOREDATADIR')
            self.DEFAULT_CONFIG_DIR = os.path.join(self.DEFAULT_ANCHORE_DATA_DIR, 'conf')
            self.DEFAULT_CONFIG_FILE = os.path.join(self.DEFAULT_CONFIG_DIR, 'config.yaml')
            self.DEFAULT_TMP_DIR = os.path.join(self.DEFAULT_ANCHORE_DATA_DIR, 'anchoretmp')
            self.DEFAULTS['tmpdir'] = self.DEFAULT_TMP_DIR
            self.DEFAULTS['anchore_data_dir'] = self.DEFAULT_ANCHORE_DATA_DIR

        self.config_dir, self.config_file = self.find_config_file()
        try:
            self.data = load_and_merge(file_path=self.config_file, defaults=self.DEFAULTS)

            self.cliargs = {}
            if cliargs:
                # store CLI arguments
                self.cliargs = cliargs

            if 'config_overrides' in self.cliargs and self.cliargs['config_overrides']:
                for override_key in self.cliargs['config_overrides']:
                    if override_key in self.data:
                        self.data[override_key] = self.cliargs['config_overrides'][override_key]

            if not os.path.exists(self.data['anchore_data_dir']):
                os.makedirs(self.data['anchore_data_dir'])

            if not os.path.exists(self.data['tmpdir']):
                os.makedirs(self.data['tmpdir'])

            #update relative to data dir if not an absolute path
            if not os.path.isabs(self.data['image_data_store']):
                self.data['image_data_store'] = os.path.join(self.data['anchore_data_dir'], self.data['image_data_store'])

            if not os.path.exists(self.data['image_data_store']):
                os.makedirs(self.data['image_data_store'])

            if not os.path.isabs(self.data['feeds_dir']):
                self.data['feeds_dir'] = os.path.join(self.data['anchore_data_dir'], self.data['feeds_dir'])

            if not os.path.isabs(self.data['user_scripts_dir']):
                self.data['user_scripts_dir'] = os.path.join(self.data['anchore_data_dir'], self.data['user_scripts_dir'])
            
            if not os.path.exists(self.data['user_scripts_dir']):
                os.makedirs(self.data['user_scripts_dir'])
        
            for d in ['analyzers', 'gates', 'queries', 'multi-queries', 'shell-utils']:
                thedir = os.path.join(self.data['user_scripts_dir'], d)
                if not os.path.exists(thedir):
                    os.makedirs(thedir)

            dc = filecmp.dircmp(os.path.join(self.data['scripts_dir'], 'shell-utils'), os.path.join(self.data['user_scripts_dir'], 'shell-utils'))
            for f in dc.left_only + dc.diff_files:
                try:
                    shutil.copy(os.path.join(self.data['scripts_dir'], 'shell-utils', f), os.path.join(self.data['user_scripts_dir'], 'shell-utils', f))
                except Exception as err:
                    raise err
            
        except Exception as err:
            self.data = None
            import traceback
            traceback.print_exc()
            raise err

    def __getitem__(self, item):
        """
        Allow dict-style access on the object itself.
        :param item:
        :return:
        """
        return self.data[item]

    def __setitem__(self, key, value):
        """
        Allow dict-style setting of values into the backing structure
        :param key:
        :param value:
        :return:
        """
        self.data[key] = value

    def __str__(self):
        return yaml.safe_dump(self.data)

    def find_config_file(self):
        thefile = ""
        thedir = ""
        if os.path.exists(self.DEFAULT_CONFIG_FILE):
            thefile = self.DEFAULT_CONFIG_FILE
            thedir = self.DEFAULT_CONFIG_DIR
        elif os.path.exists("/etc/anchore/config.yaml"):
            thefile = "/etc/anchore/config.yaml"
            thedir = "/etc/anchore"
        else:
            # config file doesn't exist, copy the example default to ~
            if not os.path.exists(self.DEFAULT_CONFIG_DIR):
                os.makedirs(self.DEFAULT_CONFIG_DIR)
            for d in os.listdir(self.EXAMPLE_CONFIG_DIR):
                if not os.path.exists('/'.join([self.DEFAULT_CONFIG_DIR, d])):
                    shutil.copy('/'.join([self.EXAMPLE_CONFIG_DIR, d]), '/'.join([self.DEFAULT_CONFIG_DIR, d]))

            thefile = self.DEFAULT_CONFIG_FILE
            thedir = self.DEFAULT_CONFIG_DIR

        return thedir, thefile

    def backup(self, destdir='/tmp'):
        dateval = datetime.datetime.now().isoformat('-')
        backupfile = os.path.join(destdir, 'anchore-backup-{0}.tar.gz'.format(dateval))

        data_dir = self.data['anchore_data_dir']
        image_dir = self.data['image_data_store']

        #module_logger.info('Backing up anchore to: %s' % backupfile)
        # Just tar up the whole thing
        with tarfile.TarFile.open(name=backupfile, mode='w:gz') as tf:
            tf.add(data_dir)
            if not data_dir in image_dir:
                # It's outside the regular data dir tree, so explicitly include
                tf.add(image_dir)
            if not data_dir in self.config_dir:
                tf.add(self.config_dir)

        return backupfile

    def restore(self, dest_root, backup_file):
        if not os.path.exists(dest_root):
            raise StandardError('Destination root dir does not exist')

        if isinstance(backup_file, str) and not os.path.exists(backup_file):
            raise StandardError('Backup file %s not found' % backup_file)

        if isinstance(backup_file, str):
            #module_logger.info('Restoring anchore from file %s to path: %s' % (backup_file, dest_root))
            with tarfile.TarFile.open(backup_file, mode='r:gz') as tf:
                tf.extractall(path=dest_root)

        else:
            #module_logger.info('Restoring anchore from file %s to path: %s' % (backup_file.name, dest_root))
            with tarfile.TarFile.open(fileobj=backup_file, mode='r:gz') as tf:
                tf.extractall(path=dest_root)

        return dest_root



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess
import stat

import anchore.anchore_utils

analyzer_name = "file_suids"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
imgid = config['imgid_full']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

#if not os.path.exists(outputdir):
#    os.makedirs(outputdir)

outfiles = {}

try:
    allfiles = {}
    if os.path.exists(unpackdir + "/anchore_allfiles.json"):
        with open(unpackdir + "/anchore_allfiles.json", 'r') as FH:
            allfiles = json.loads(FH.read())
    else:
        fmap, allfiles = anchore.anchore_utils.get_files_from_path(unpackdir + "/rootfs")
        with open(unpackdir + "/anchore_allfiles.json", 'w') as OFH:
            OFH.write(json.dumps(allfiles))

    # fileinfo
    for name in allfiles.keys():
        if allfiles[name]['mode'] & stat.S_ISUID:
            outfiles[name] = oct(stat.S_IMODE(allfiles[name]['mode']))

except Exception as err:
    print "ERROR: " + str(err)

if outfiles:
    ofile = os.path.join(outputdir, 'files.suids')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, outfiles)

sys.exit(0)



import os
import click
import subprocess
import docker
import sys
from anchore.configuration import AnchoreConfiguration
from anchore.version import version as anchore_version

import analyzer
#import explorer
import query
import audit
#import subscriptions
import system
#import synchronizer
import logs
import toolbox
import login
import feeds
import anchore.anchore_image_db, anchore.anchore_utils

from anchore.util import contexts
from .common import init_output_format, anchore_print_err, extended_help_option


main_extended_help="""

Anchore is a tool for analyzing, querying, and curating container
images to deliver the transparency, predictability, and control
necessary to use containers in production. Anchore is composed of a
toolset that runs locally on a host as well as a web service that
monitors the container ecosystem and provides inputs to the local
toolset.

The tool has capabilities to populate a registry, run analysis,
explore/query the analysis results (including custom queries and
checks), and run policy-based gate-functions against container images
to help an ops team decide if a container should go into the CI/CD
pipeline and on to production based on attributes of the Dockerfile,
built image, or both.

After installation, the first command run should be: 'anchore feeds
list' to initialize the system and load feed data.

Feeds are different types of data that the anchore web service makes
available for certain container image queries and scans.  At any point
in time, you can use the 'anchore feeds' commands to sync the latest
data from the anchore web service, subscribe and/or unsubscribe from
anchore feeds, or get a list of available feeds. Except during feeds
operations, no network connectivity is required by anchore to run
analysis and query images.

Configuration Files:

Anchore configuration files are automatically installed if not found when looking in-order at:
\b
$HOME/.anchore/conf
/etc/anchore

A default install will copy the files to $HOME/.anchore/conf/, but they may be manually put in /etc/anchore for a system global config.

* config.yaml - main configuration file. Used to override default values. Anchore will search for it in
$HOME/.anchore/conf, then in /etc/anchore. If not found in either place, a new one is initilized and copied to $HOME/.anchore

* anchore_gate.policy - The global gate policy file (see anchore-gate(1) for more help on gates).


High-level example flows:

Initialize the system and sync the by-default subscribed feed 'vulnerabilties':

\b
anchore feeds list
anchore feeds sync

Analyze an image

docker pull nginx:latest
anchore analyze --image nginx:latest --imagetype base

Generate a summary report on all analyzed images

anchore audit report

Check gate output for nginx:latest:

anchore gate --image nginx:latest
"""


@click.group()
@click.option('--verbose', is_flag=True, help='Enable verbose output to stderr.')
@click.option('--debug', is_flag=True, help='Developer debug output to stderr.')
@click.option('--quiet', is_flag=True, help='Only errors to stderr, no status messages.')
@click.option('--json', is_flag=True, help='Output formatted json to stdout.')
@click.option('--plain', is_flag=True, help='Output formatted scriptable text to stdout.')
@click.option('--html', is_flag=True, help='Output formatted HTML table to stdout.')
@click.option('--config-override', help='Override an anchore configuration option (can be used multiple times).', metavar='<config_opt>=<config_value>', multiple=True)

@click.version_option(version=anchore_version)
@click.pass_context
@extended_help_option(extended_help=main_extended_help)
def main_entry(ctx, verbose, debug, quiet, json, plain, html, config_override):
    """
    Anchore is a tool to analyze, query, and curate container images. The options at this top level
    control stdout and stderr verbosity and format.

    After installation, the first command run should be: 'anchore feeds
    list' to initialize the system and load feed data.


    High-level example flows:

    Initialize the system and sync the by-default subscribed feed 'vulnerabilties':

    \b
    anchore feeds list
    anchore feeds sync

    Analyze an image

    docker pull nginx:latest
    anchore analyze --image nginx:latest --imagetype base

    Generate a summary report on all analyzed images

    anchore audit report

    Check gate output for nginx:latest:

    anchore gate --image nginx:latest
    """
    # Load the config into the context object
    logfile = None
    debug_logfile = None

    try:
        try:
            config_overrides = {}
            if config_override:
                for el in config_override:
                    try:
                        (key, val) = el.split('=')
                        if not key or not val:
                            raise Exception("could not split by '='")
                        config_overrides[key] = val
                    except:
                        click.echo("Error: specified --config_override param cannot be parsed (should be <config_opt>=<value>): " + str(el))
                        exit(1)

            args = {'verbose': verbose, 'debug': debug, 'json': json, 'plain': plain, 'html': html, 'quiet': quiet, 'config_overrides':config_overrides}
            anchore_conf = AnchoreConfiguration(cliargs=args)
        except Exception as err:
            click.echo("Error setting up/reading Anchore configuration", err=True)
            click.echo("Info: "+str(err), err=True)
            import traceback
            traceback.print_exc()
            sys.exit(1)
        try:
            logfile = anchore_conf.data['log_file'] if 'log_file' in anchore_conf.data else None
            debug_logfile = anchore_conf.data['debug_log_file'] if 'debug_log_file' in anchore_conf.data else None
        except Exception, e:
            click.echo(str(e))

        ctx.obj = anchore_conf

    except:
        if ctx.invoked_subcommand != 'system':
            click.echo('Expected, but did not find configuration file at %s' % os.path.join(AnchoreConfiguration.DEFAULT_CONFIG_FILE), err=True)
            exit(1)

    try:
        init_output_format(json, plain, debug, verbose, quiet, log_filepath=logfile, debug_log_filepath=debug_logfile)
    except Exception, e:
        click.echo('Error initializing logging: %s' % str(e))
        exit(2)

    if not anchore_pre_flight_check(ctx):
        anchore_print_err("Error running pre-flight checks")
        exit(1)

    try:
        if not anchore.anchore_utils.anchore_common_context_setup(ctx.obj):
            anchore_print_err("Error setting up common data based on configuration")
            exit(1)
    except ValueError as err:
        print "ERROR: " + str(err)
        exit(1)

main_entry.add_command(system.system)
main_entry.add_command(query.query)
main_entry.add_command(audit.audit)
main_entry.add_command(analyzer.analyze)
main_entry.add_command(analyzer.gate)
main_entry.add_command(toolbox.toolbox)
main_entry.add_command(login.login)
main_entry.add_command(login.logout)
main_entry.add_command(login.whoami)
main_entry.add_command(feeds.feeds)

def anchore_pre_flight_check(ctx):
    # helper checks
    try:
        subcommand = ctx.invoked_subcommand
        config = ctx.obj.data
    except:
        return(False)
        
    if subcommand in ['explore', 'gate', 'analyze']:

        # check for some shellouts for analyzers
        try:
            cmd = ['dpkg-query', '--version']
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except Exception as err:
            anchore_print_err("Anchore requires dpkg libs and commands")
            return(False)

        try:
            from rpmUtils.miscutils import splitFilename
            cmd = ['rpm', '--version']
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except Exception as err:
            anchore_print_err("Anchore requires yum/rpm libs and commands")
            return(False)

    if subcommand in ['explore', 'gate', 'analyze', 'toolbox']:
        # check DB readiness
        try:
            db = anchore.anchore_image_db.load(driver=config['anchore_db_driver'], config=config)
        except Exception as err:
            anchore_print_err("Could not set up connection to Anchore DB")
            return(False)

    return(True)



#!/usr/bin/env python
import sys
import os
import re
import traceback

import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <distro> <distro> ...\nhelp: Shows the distro/version that the container image is based on - use 'all' for all or specify a string to filter the results.")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    print "Query requires input: all | <distro> <distro> ..."

outlist = list()
outlist.append(["Image_Id", "Repo_Tag", "Distro", "Version"])

try:
    # handle the good case, something is found resulting in data matching the required columns

    distrodict = anchore.anchore_utils.get_distro_from_imageId(config['imgid'])

    tags = "none"
    if config['meta']['humanname']:
        tags = config['meta']['humanname']

    distro = "unknown"
    if 'DISTRO' in distrodict:
        distro = distrodict['DISTRO']
        
    distrovers = "unknown"
    if 'DISTROVERS' in distrodict:
        distrovers = distrodict['DISTROVERS']

    match=False
    for d in config['params']:
        if d == 'all' or d == 'ALL':
            match = True
            break
        if d == distro or d == distro+':'+distrovers:
            match = True
            break

    if match:
        outlist.append([config['meta']['shortId'], tags, distro, distrovers])

except Exception as err:
    print str(err)
    traceback.print_exc()
    pass

# handle the no match case
if len(outlist) < 1:
    pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

sys.exit(0)






def load(driver=None, config={}):
    dbobj = None
    for base in ["anchore.anchore_image_db", "anchore-modules.anchore_image_db"]:
        try:
            module = __import__(base, fromlist=[driver])
            c = getattr(module, driver)
            dbobj = c.load(config=config)
            break
        except AttributeError as err:
            # didn't find the driver, yet            
            pass
        except Exception as err:
            raise err
            
    if not dbobj:
        raise Exception ("DB driver not found: " + str(driver))

    return(dbobj)



import sys
import os
import getpass
import click

from anchore.cli.common import anchore_print, anchore_print_err
from anchore.util import contexts
from anchore import anchore_auth


@click.command(name='login', short_help='Log in to the Anchore service.')
@click.pass_obj
def login(anchore_config):
    """
    Log into Anchore service using your username/password from anchore.io.
    """
    config = anchore_config
    ecode = 0

    try:
        if os.getenv('ANCHOREUSER'):
            anchore_print("Using user from environment (ANCHOREUSER)")
            username = os.getenv('ANCHOREUSER')
        else:
            username = raw_input("Username: ")

        if os.getenv('ANCHOREPASS'):
            anchore_print("Using password from environment (ANCHOREPASS)")
            password = os.getenv('ANCHOREPASS')
        else:
            password = getpass.getpass("Password: ")
            
        aa = contexts['anchore_auth']

        new_anchore_auth = anchore_auth.anchore_auth_init(username, password, aa['auth_file'], aa['client_info_url'], aa['token_url'], aa['conn_timeout'], aa['max_retries'])
        rc, ret = anchore_auth.anchore_auth_refresh(new_anchore_auth)
        if not rc:
            anchore_print("Failed to log in: check your username/password and try again!")
            anchore_print("Message from server: " + ret['text'])
        else:
            contexts['anchore_auth'].update(new_anchore_auth)
            anchore_print("Login successful.")

    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)


@click.command(name='logout', short_help='Log out of the Anchore service.')
@click.pass_obj
def logout(anchore_config):
    """
    Log out of Anchore service
    """
    ecode = 0
    try:
        aa = contexts['anchore_auth']
        if aa:
            anchore_auth.anchore_auth_invalidate(aa)
            if 'auth_file' in aa:
                os.remove(aa['auth_file'])
        print "Logout successful."
    except Exception as err:
        anchore_print_err('operation failed')
        ecode = 1

    sys.exit(ecode)


@click.command(name='whoami', short_help='Show user data for current logged-in user if available')
@click.pass_obj
def whoami(anchore_config):
    """
    Show user data for current user if available
    :param anchore_config:
    :return:
    """
    ecode = 0
    try:
        aa = contexts['anchore_auth']
        if aa and 'username' in aa and 'password' in aa:
            info = {'Current user': aa['user_info'] if aa['user_info'] else 'anonymous'}

            anchore_print(info, do_formatting=True)
        else:
            anchore_print_err('No anchore auth context found. Cannot get user info. Try logging in first')
            ecode = 1

    except Exception as err:
        anchore_print_err('Cannot get user info')
        ecode = 1

    sys.exit(ecode)

        




import json
import re
import os
import shutil
import time
import logging
import tarfile

from anchore import anchore_utils
import anchore_image_db_base

def load(config={}):
    return(AnchoreImageDB_FS(config=config))

class AnchoreImageDB_FS(anchore_image_db_base.AnchoreImageDB):
    _db_metadata_filename = 'anchore_db_meta.json'

    def __init__(self, config={}):
        anchore_image_db_base.AnchoreImageDB.__init__(self, config)
        if not self.config:
            raise Exception("invalid config passed to driver init")

        try:
            dbconfig = config['anchore_db_driver_config']
            try:
                dbdir = dbconfig['db_dir']
                feeddir = dbconfig['feed_dir']
            except:
                dbdir = None
                feeddir = None
        except:
            dbdir = None
            feeddir = None
            dbconfig = None

        try:
            basedir = config['anchore_data_dir']
        except:
            basedir = None

        try:
            olddatadir = config['image_data_store']
        except:
            olddatadir = None

        try:
            oldfeeddir = config['feeds_dir']
        except:
            oldfeeddir = None

        # for image data
        if dbdir and basedir:
            # try to construct a dir off the db config options
            if not os.path.isabs(dbdir):
                imagerootdir = os.path.join(basedir, dbdir)
            else:
                imagerootdir = dbdir
        elif olddatadir:
            # use the old method
            imagerootdir = olddatadir
        else:
            imagerootdir = None

        # for feeds
        if feeddir and basedir:
            # try to construct a dir off the db config options
            if not os.path.isabs(feeddir):
                feedrootdir = os.path.join(basedir, feeddir)
            else:
                feedrootdir = feeddir
        elif oldfeeddir:
            # use the old method
            feedrootdir = oldfeeddir
        else:
            feedrootdir = None

        self._logger.debug("using directory for anchore image data: " + str(imagerootdir))
        self._logger.debug("using directory for anchore feed data: " + str(feedrootdir))

        self.imagerootdir = imagerootdir
        self.feedrootdir = feedrootdir
        
        try:
            for d in [self.imagerootdir, self.feedrootdir]:
                if not os.path.exists(d):
                    try:
                        os.makedirs(d)
                    except:
                        raise Exception("anchore DB: data dir does not exist and cannot be created "+str(d))

            # check that the software and DB are same version before continuing
            dbmetafile = '/'.join([self.imagerootdir, "anchore_db_meta.json"])
            if not os.path.exists(dbmetafile):
                with open(dbmetafile, 'w') as OFH:
                    OFH.write(json.dumps({}))

            update = False

            with open(dbmetafile, 'r') as FH:
                json_dict = json.loads(FH.read())

            if 'anchore_version' not in json_dict:
                json_dict['anchore_version'] = self.anchore_version_string
                update = True
            elif json_dict['anchore_version'] != self.anchore_version_string:
                json_dict['anchore_version'] = self.anchore_version_string
                update = True

            if 'anchore_db_version' not in json_dict:
                json_dict['anchore_db_version'] = self.anchore_db_version_string
                update = True
            elif json_dict['anchore_db_version'] != self.anchore_db_version_string:
                # Check major version
                local_db_version = json_dict['anchore_db_version']
                db_version_tuple = tuple(local_db_version.split('.'))
                code_version_tuple = tuple(self.anchore_db_version_string.split('.'))
                if db_version_tuple[0] < code_version_tuple[0]:
                    raise ValueError("Anchore DB version ("+local_db_version + ") cannot be automatically upgraded to this Anchore software version (" + self.anchore_version_string + ") Please downgrade Anchore to compatible version or remove your Anchore DB and start from a clean install.")
                else:
                    json_dict['anchore_db_version'] = self.anchore_db_version_string
                    update = True

            if update:
                with open(dbmetafile, 'w') as OFH:
                    OFH.write(json.dumps(json_dict))

        except ValueError as err:
            raise err
        except Exception as err:
            raise err

        self.initialized = True        

    @staticmethod
    def exists(path, with_expected_db_version=None):
        """
        Does the db exist, optionally with the expected version
        :param path:
        :param with_expected_db_version:
        :return:
        """
        dbmetafile = os.path.join(path, AnchoreImageDB._db_metadata_filename)

        if not os.path.exists(path) or not os.path.exists(dbmetafile):
            return False

        if with_expected_db_version:
            with open(dbmetafile, 'r') as f:
                json_dict = json.load(f)

            if json_dict['anchore_db_version'] != with_expected_db_version:
                return False
        else:
            return True

    def is_image_present(self, imageId, imagelist=None):
        thefile = '/'.join([self.imagerootdir, imageId])
        if not os.path.exists(thefile):
            return(False)
        return(True)

    def is_image_analyzed(self, imageId):
        amanifest = self.load_analyzer_manifest(imageId)
        if not amanifest:
            return(False)
        else:
            for a in amanifest.keys():
                try:
                    if amanifest[a]['status'] != "SUCCESS":
                        self._logger.error("analyzer ("+str(a)+") not executed/failed")
                        return(False)
                except Exception as err:
                    return(False)
        return(True)

    def get_image_list(self):
        ret = {}
        for d in os.listdir(self.imagerootdir):
            if os.path.exists(os.path.join(self.imagerootdir, d, "analyzers.done")):
                t = self.load_image_report(d)
                ret[d] = list(set(t['anchore_all_tags']) | set(t['anchore_current_tags']))
        return(ret)

    def load_all_images(self):
        ret = {}
        
        for d in os.listdir(self.imagerootdir):
            try:
                result = self.load_image(d)
                if result['meta']['imageId'] not in ret:
                    ret[result['meta']['imageId']] = result
            except Exception as err:
                pass
            
        return(ret)

    def load_all_images_iter(self):
        for d in os.listdir(self.imagerootdir):
            try:
                result = self.load_image(d)
                r = (result['meta']['imageId'], result)
                yield r
            except Exception as err:
                pass

    def delete_image(self, imageId):
        imagedir = '/'.join([self.imagerootdir, imageId])
        if os.path.exists(imagedir):
            shutil.rmtree(imagedir)
            
    def load_image(self, imageId):
        return(self.load_image_report(imageId));

    def load_image_new(self, imageId):
        ret = {}

        ret['image_report'] = self.load_image_report(imageId)
        ret['analysis_report'] = self.load_analysis_report(imageId)
        ret['analyzer_manifest'] = self.load_analyzer_manifest(imageId)
        ret['gates_report'] = self.load_gates_report(imageId)
        ret['gates_eval_report'] = self.load_gates_eval_report(imageId)

        return(ret)

    def create_image(self, imageId):
        return(self.make_image_structure(imageId))

    def make_image_structure(self, imageId):
        imgdir = os.path.join(self.imagerootdir, imageId)
        subdirs = ['analyzer_output', 'analyzer_output_extra', 'analyzer_output_user', 'gates_output', 'image_output', 'reports']
        
        if not os.path.exists(imgdir):
            os.makedirs(imgdir)

        for d in subdirs:
            if not os.path.exists(os.path.join(imgdir, d)):
                os.makedirs(os.path.join(imgdir, d))
        return(True)

    def save_image_new(self, imageId, report=None):
        # setup dir structure
        self.make_image_structure(imageId)

        if not report:
            report = load_image_new(imageId)

        # store the reports
        self.save_image_report(imageId, report['image_report'])
        self.save_analysis_report(imageId, report['analysis_report'])
        self.save_analyzer_manifest(imageId, report['analyzer_manifest'])
        self.save_gates_report(imageId, report['gates_report'])
        self.save_gates_eval_report(imageId, report['gates_eval_report'])

        # populate the analyzer_outputs
        for module_name in report['analysis_report'].keys():
            for module_value in report['analysis_report'][module_name].keys():
                for module_type in ['base', 'extra', 'user']:
                    if module_type in report['analysis_report'][module_name][module_value]:
                        adata = report['analysis_report'][module_name][module_value][module_type]
                        if adata:
                            self.save_analysis_output(imageId, module_name, module_value, adata, module_type=module_type)
                            
        # populate gates outputs
        for gname in report['gates_report'].keys():
            self.save_gate_output(imageId, gname, report['gates_report'][gname])

        # populate image metadata
        thedir = os.path.join(self.imagerootdir, imageId, "image_output", "image_info")
        if not os.path.exists(thedir):
            os.makedirs(thedir)

        thefile = os.path.join(thedir, "image.meta")
        anchore_utils.write_kvfile_fromdict(thefile, report['image_report']['meta'])

        thefile = os.path.join(thedir, "Dockerfile")
        anchore_utils.write_plainfile_fromstr(thefile, report['image_report']['dockerfile_contents'])

        return(True)
                

    def load_analysis_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/analysis_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_analysis_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/analysis_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def list_analysis_outputs(self, imageId):
        aret = {}
        amanifest = self.load_analyzer_manifest(imageId)
        for a in amanifest.keys():
            if 'analyzer_outputs' in amanifest[a]:
                for record in amanifest[a]['analyzer_outputs']:
                    module_name = record['module_name']
                    module_value = record['module_value']
                    if module_name not in aret:
                        aret[module_name] = {}
                    aret[module_name][module_value] = True
        return(aret)

    def load_analyzer_manifest(self, imageId):
        ret = {}
        thefile = os.path.join(self.imagerootdir, imageId, 'analyzers.done')
        if os.path.exists(thefile):
            with open(thefile, 'r') as FH:
                try:
                    ret = json.loads(FH.read())
                except:
                    ret = {}
        return(ret)

    def save_analyzer_manifest(self, imageId, data):
        thefile = os.path.join(self.imagerootdir, imageId, 'analyzers.done')
        if data:
            with open(thefile, 'w') as FH:
                FH.write(json.dumps(data))

    def load_analysis_output(self, imageId, module_name, module_value, module_type=None):
        ret = {}

        if not module_type or module_type == 'base':
            thefile = '/'.join([self.imagerootdir, imageId, "analyzer_output", module_name, module_value])
        else:
            thefile = '/'.join([self.imagerootdir, imageId, "analyzer_output_"+module_type, module_name, module_value])

        if os.path.exists(thefile):
            if os.path.isfile(thefile):
                ret = anchore_utils.read_kvfile_todict(thefile)
            elif os.path.isdir(thefile):
                ret = thefile
                import tarfile, io
                try:
                    TFH = io.BytesIO()
                    tar = tarfile.open(fileobj=TFH, mode='w:gz', format=tarfile.PAX_FORMAT)
                    for l in os.listdir(thefile):
                        tarfile = os.path.join(thefile, l)
                        tar.add(tarfile)
                    tar.close()
                    TFH.seek(0)
                    ret = TFH
                except Exception as err:
                    import traceback
                    traceback.print_exc()
                    print str(err)

        return(ret)

    def save_analysis_output(self, imageId, module_name, module_value, data, module_type=None, directory_data=False):
        if not module_type or module_type == 'base':
            odir = '/'.join([self.imagerootdir, imageId, "analyzer_output", module_name])
        else:
            odir = '/'.join([self.imagerootdir, imageId, "analyzer_output_"+module_type, module_name])

        if not directory_data:
            thefile = '/'.join([odir, module_value])
            if not os.path.exists(odir):
                os.makedirs(odir)

            return(anchore_utils.write_kvfile_fromdict(thefile, data))
        else:
            if os.path.isdir(data):
                if os.path.isdir(odir):
                    shutil.rmtree(odir)
                os.makedirs(odir)
                shutil.move(data, odir)

    def load_gates_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/gates_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_gates_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/gates_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    
    def load_gate_output(self, imageId, gate_name):
        thedir = os.path.join(self.imagerootdir, imageId, 'gates_output')
        thefile = os.path.join(thedir, gate_name)
        return(anchore_utils.read_plainfile_tolist(thefile))

    def list_gate_outputs(self, imageId):
        ret = list()
        thedir = os.path.join(self.imagerootdir, imageId, 'gates_output')
        if not os.path.exists(thedir):
            return(ret)
        for d in os.listdir(thedir):
            if re.match(".*\.eval$", d) or re.match(".*\.help$", d):
                continue

            if d not in ret:
                ret.append(d)
        return(ret)

    def save_gate_output(self, imageId, gate_name, data):
        thedir = os.path.join(self.imagerootdir, imageId, 'gates_output')
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = os.path.join(thedir, gate_name)
        return(anchore_utils.write_plainfile_fromlist(thefile, data))

    def save_gate_help_output(self, gate_help):
        thedir = os.path.join(self.imagerootdir)
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = os.path.join(thedir, "gates_info.json")
        rc = anchore_utils.update_file_jsonstr(json.dumps(gate_help), thefile)
        return(rc)

    def save_gate_eval_output(self, imageId, gate_name, data):
        thedir = os.path.join(self.imagerootdir, imageId, "gates_output")
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = os.path.join(thedir, gate_name+".eval")
        return(anchore_utils.write_plainfile_fromlist(thefile, data))

    def del_gate_eval_output(self, imageId, gate_name):
        thefile = os.path.join(self.imagerootdir, imageId, "gates_output", imageId, gate_name)
        if os.path.exists(thefile):
            try:
                os.remove(thefile)
            except:
                pass
        return(True)

    def load_gates_eval_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/gates_eval_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_gates_eval_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/gates_eval_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def load_gate_policy(self, imageId):
        thefile = os.path.join(self.imagerootdir, imageId, 'anchore_gate.policy')
        return(anchore_utils.read_plainfile_tolist(thefile))

    def save_gate_policy(self, imageId, data):
        thedir = os.path.join(self.imagerootdir, imageId)
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = os.path.join(thedir, "anchore_gate.policy")
        return(anchore_utils.write_plainfile_fromlist(thefile, data))

    def del_gate_policy(self, imageId):
        thefile = os.path.join(self.imagerootdir, imageId, 'anchore_gate.policy')
        try:
            os.remove(thefile)
        except:
            pass
        return(True)

    def load_gate_whitelist(self, imageId):
        thefile = os.path.join(self.imagerootdir, imageId, 'anchore_gate.whitelist')
        return(anchore_utils.read_plainfile_tolist(thefile))

    def save_gate_whitelist(self, imageId, data):
        thedir = os.path.join(self.imagerootdir, imageId)
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = os.path.join(thedir, 'anchore_gate.whitelist')
        return(anchore_utils.write_plainfile_fromlist(thefile, data))

    def load_image_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/image_report.json"
        if not os.path.exists(thefile):
            return({})

        ret = {}
        try:
            with open(thefile, 'r') as FH:
                ret = json.loads(FH.read())
        except Exception as err:
            self._logger.debug("image report json found ("+str(thefile)+"), but failed to load for imageId ("+str(imageId)+") - exception: " + str(err))
            ret = {}

        return(ret)

    def save_image_report(self, imageId, report):
        # populate image metadata
        thedir = os.path.join(self.imagerootdir, imageId, "image_output", "image_info")
        if not os.path.exists(thedir):
            os.makedirs(thedir)

        thefile = os.path.join(thedir, "image.meta")
        if 'meta' in report:
            anchore_utils.write_kvfile_fromdict(thefile, report['meta'])

        # save the report itself
        date = str(int(time.time()))
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/image_report.json"

        if os.path.exists(thefile):
            oldreport = self.load_image_report(imageId)
            if 'tag_history' in oldreport:
                report['tag_history'] = list(oldreport['tag_history'])
            else:
                report['tag_history'] = list()

            if 'anchore_current_tags' not in oldreport:
                oldreport['anchore_current_tags'] = list([date, report['anchore_current_tags']])

            diff = list(set(oldreport['anchore_current_tags']).symmetric_difference(set(report['anchore_current_tags'])))
            if len(diff) > 0:
                # there is a difference between stored tags and new tags
                report['tag_history'].append([date, oldreport['anchore_current_tags']])
        
        if 'tag_history' not in report:
            report['tag_history'] = list()

        if len(report['tag_history']) <= 0:
            report['tag_history'].append([date, report['anchore_current_tags']])

        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def save_files(self, imageId, namespace, rootfsdir, files):
        def tarfilter(member):
            subber = re.sub("^/*", "", rootfsdir)
            subber = re.sub("/*$", "", subber)
            finalstr = '/'.join(['imageroot', re.sub("^"+re.escape(subber)+"/*", "", member.name)])
            member.name = finalstr
            return(member)

        thedir = os.path.join(self.imagerootdir, imageId, "file_store", namespace)
        if not os.path.exists(thedir):
            os.makedirs(thedir)

        tar = tarfile.open('/'.join([thedir, 'stored_files.tar.gz']), mode='w:gz', format=tarfile.PAX_FORMAT)
        for thefile in files:
            if os.path.exists(thefile):
                print "INFO: storing file: " + str(thefile)
                tar.add(thefile, filter=tarfilter)
            else:
                print "WARN: could not find file ("+str(thefile)+") in image: skipping store"
        tar.close()
        return(True)

    def save_files_tarfile(self, imageId, namespace, tarfile):
        if not os.path.exists(tarfile):
            return(False)

        thedir = os.path.join(self.imagerootdir, imageId, "file_store", namespace)
        thefile = os.path.join(thedir, "stored_files.tar.gz")
        if not os.path.exists(thedir):
            os.makedirs(thedir)

        shutil.copy(tarfile, thefile)

        return(True)

    def load_files_tarfile(self, imageId, namespace):
        thedir = os.path.join(self.imagerootdir, imageId, "file_store", namespace)
        thefile = os.path.join(thedir, 'stored_files.tar.gz')
        if not os.path.exists(thefile):
            thefile = os.path.join(self.imagerootdir, imageId, 'analyzer_output', namespace, 'file_cache', 'stored_files.tar.gz')

        if not os.path.exists(thefile):
            return(False)
        return(thefile)

    def load_files_namespaces(self, imageId):
        namespaces = list()
        thedir = os.path.join(self.imagerootdir, imageId, "file_store")
        if os.path.exists(thedir):
            for d in os.listdir(thedir):
                namespaces.append(d)
        return(namespaces)

    def load_files_metadata(self, imageId, namespace):
        thedir = os.path.join(self.imagerootdir, imageId, "file_store", namespace)
        thefile = os.path.join(thedir, 'stored_files.tar.gz')
        if not os.path.exists(thefile):
            thefile = os.path.join(self.imagerootdir, imageId, 'analyzer_output', namespace, 'file_cache', 'stored_files.tar.gz')

        if not os.path.exists(thefile):
            allfiles = {}
        else:
            allfiles = anchore_utils.get_files_from_tarfile(thefile)

        return(allfiles)

    def del_files_cache(self, imageId, namespace):
        return(True)

    def load_feedmeta(self):
        ret = {}

        basedir = self.feedrootdir
        feedfile = os.path.join(basedir, "feedmeta.json")
        feedmeta = {}
        if os.path.exists(feedfile):
            with open(feedfile, 'r') as FH:
                ret = json.loads(FH.read())

        return(ret)

    def save_feedmeta(self, feedmeta):
        basedir = self.feedrootdir
        feedfile = os.path.join(basedir, "feedmeta.json")
        if feedmeta:
            with open(feedfile, 'w') as OFH:
                OFH.write(json.dumps(feedmeta))
            return(True)
        return(False)

    def create_feed(self, feed):
        basedir = self.feedrootdir
        thedir = os.path.join(basedir, feed)
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        return(True)

    def create_feedgroup(self, feed, group):
        basedir = self.feedrootdir
        thedir = os.path.join(basedir, feed, group)
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        return(True)
    
    def delete_feed(self, feed):
        basedir = self.feedrootdir
        thedir = os.path.join(basedir, feed)
        if os.path.exists(thedir):
            shutil.rmtree(thedir)

    def save_feed_group_data(self, feed, group, datafilename, data):
        basedir = self.feedrootdir
        thedir = os.path.join(basedir, feed, group)
        if not os.path.exists(thedir):
            os.makedirs(thedir)

        thefile = os.path.join(thedir, datafilename)

        jsondata = json.dumps(data).encode('utf8')

        with open(thefile, 'w') as OFH:
            OFH.write(jsondata)

        return(True)

    def load_feed_group_data(self, feed, group, datafilename):
        ret = []
        basedir = self.feedrootdir
        thefile = os.path.join(basedir, feed, group, datafilename)

        if os.path.exists(thefile):
            with open(thefile, 'r') as FH:
                ret = json.loads(FH.read().decode('utf8'))

        return(ret)

    def delete_feed_group_data(self, feed, group, datafilename):
        ret = True
        basedir = self.feedrootdir
        thefile = os.path.join(basedir, feed, group, datafilename)
        
        if os.path.exists(thefile):
            try:
                os.remove(thefile)
            except:
                ret = False

        return(ret)
        



#!/usr/bin/env python

import sys
import os
import re
import json
import traceback

import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: all | onlyimage ...\nhelp: shows dockerfile lines, associated layer IDs, and layer sizes.  If 'onlyimage' is given, layers inherited from parent images are not displayed.")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    print "Query requires input: all|onlyimage ..."


warns = list()
outlist = list()

showall = True
if config['params'][0] == 'onlyimage':
    outlist.append(["Image_Id", "Repo_Tags", "Layer_Id", "Layer_Size_Bytes", "Dockerfile_Line"])
    showall = False
else:
    outlist.append(["Image_Id", "Repo_Tags", "Layer_Id", "Layer_Size_Bytes", "Dockerfile_Line", "Inherited_From_Parent"])    

try:
    ftreetypes = {}

    idata = anchore.anchore_utils.load_image_report(config['imgid'])
    for fid in idata['familytree']:
        fdata = anchore.anchore_utils.load_image_report(fid)
        ftreetypes[fid] = fdata['meta']['usertype']

    ddata = anchore.anchore_utils.load_analysis_output(config['imgid'], 'layer_info', 'layers_to_dockerfile')
    ddata_list = json.loads(ddata['dockerfile_to_layer_map'])
    inherited = False
    for record in ddata_list:
        layer = record['layer']
        if not inherited:
            if layer != config['imgid']:
                (fromline, fromid) = anchore.anchore_utils.discover_from_info(idata['dockerfile_contents'])
                if fromid and layer == fromid:
                    inherited = True
                elif layer in ftreetypes:
                    if ftreetypes[layer] and ftreetypes[layer] != "none":
                        inherited = True
                
                
        dfileline = ' '.join(record['dockerfile_line'].split())
        if not dfileline:
            dfileline = "N/A"

        dfileline = re.sub("^/bin/sh -c", "RUN", dfileline)

        if showall:
            outlist.append([config['meta']['shortId'], config['meta']['humanname'], record['layer'], record['layer_sizebytes'], dfileline, str(inherited)])
        else:
            if not inherited:
                outlist.append([config['meta']['shortId'], config['meta']['humanname'], record['layer'], record['layer_sizebytes'], dfileline])
                
    pass

except Exception as err:
    # handle the case where something wrong happened
    import traceback
    traceback.print_exc()
    warns.append("query error: "+str(err))
    pass

# handle the no match case
if len(outlist) < 1:
    #outlist.append(["NOMATCH", "NOMATCH", "NOMATCH"])
    pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

sys.exit(0)






import os
import json
import requests
import base64
import urllib
import logging

_logger = logging.getLogger(__name__)


def anchore_auth_init(username, password, auth_file, client_info_url, token_url, conn_timeout, max_retries):
    if not username or not password or not auth_file or not client_info_url or not token_url or not conn_timeout or not max_retries:
        return (False)

    anchore_auth = {'username': '',
                    'password': '',
                    'max_retries': int(max_retries),
                    'conn_timeout': int(conn_timeout),
                    'client_info_url': client_info_url,
                    'token_url': token_url,
                    'auth_file': auth_file,
                    'client_info': {},
                    'token_info': {},
                    'user_info': {},
                    }

    if os.path.exists(auth_file):
        try:
            with open(auth_file, 'r') as FH:
                loaded_anchore_auth = json.loads(FH.read())
                anchore_auth.update(loaded_anchore_auth)
        except:
            pass

    if anchore_auth['username'] != username or anchore_auth['password'] != password:
        anchore_auth['username'] = username
        anchore_auth['password'] = password
        anchore_auth_invalidate(anchore_auth)

    if not anchore_auth_save(anchore_auth, auth_file):
        _logger.error("could not save authentication details (" + auth_file + ")")
        return (False)

    return (anchore_auth)


def get_current_user_info(anchore_auth):
    """
    Return the metadata about the current user as supplied by the anchore.io service. Includes permissions and tier access.

    :return: Dict of user metadata
    """

    user_url = anchore_auth['client_info_url'] + '/' + anchore_auth['username']
    user_timeout = 60
    retries = 3
    result = requests.get(user_url, headers={'x-anchore-password': anchore_auth['password']})
    if result.status_code == 200:
        user_data = json.loads(result.content)
    else:
        raise requests.HTTPError('Error response from service: {}'.format(result.status_code))
    return user_data


def anchore_auth_invalidate(anchore_auth):
    if 'client_info' in anchore_auth:
        anchore_auth['client_info'] = {}

    if 'token_info' in anchore_auth:
        anchore_auth['token_info'] = {}

    if 'user_info' in anchore_auth:
        anchore_auth['user_info'] = {}

    return (True)


def anchore_auth_save(anchore_auth, auth_file):
    if not anchore_auth or not auth_file:
        return (False)

    # print "saving new anchore auth"
    try:
        with os.fdopen(os.open(auth_file, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0600), 'w') as OFH:
            OFH.write(json.dumps(anchore_auth))
    except:
        return (False)

    return (True)


def anchore_auth_refresh(anchore_auth, forcerefresh=False):
    ret = {'success': False, 'text': "", 'status_code': 0}
    if not anchore_auth:
        ret['text'] = json.dumps("no anchore_auth token given as input")
        return (False, ret)

    new_anchore_auth = {}
    new_anchore_auth.update(anchore_auth)

    username = anchore_auth['username']
    password = anchore_auth['password']
    url_username = urllib.quote_plus(username)
    url_password = urllib.quote_plus(password)
    client_info_url = anchore_auth['client_info_url']
    token_url = anchore_auth['token_url']
    client_info = anchore_auth['client_info']
    token_info = anchore_auth['token_info']
    auth_file = anchore_auth['auth_file']
    user_info = anchore_auth['user_info']
    conn_timeout = int(anchore_auth['conn_timeout'])

    if not client_info:
        # get client info
        url = client_info_url + "/" + username
        headers = {'x-anchore-password': password}
        try:
            r = requests.get(url, headers=headers, timeout=conn_timeout)
        except:
            # print "request timed out"
            ret['text'] = json.dumps("connection timed out: increase anchore_auth_conn_timeout higher or try again")
            return (False, ret)

        ret['text'] = r.text
        ret['status_code'] = r.status_code

        if r.status_code == 200:
            new_anchore_auth['client_info'] = json.loads(r.text)['clients'][0]
            client_info = new_anchore_auth['client_info']
            ret['success'] = True
        elif r.status_code == 401:
            # print "bad username/password!"
            return (False, ret)
        else:
            # print "unknown login error: "
            return (False, ret)
    else:
        pass
        # print "skipping client_info get"

    b64bearertok = base64.b64encode(client_info['client_id'] + ":" + client_info['client_secret'])

    if not token_info:
        # get a token set
        payload = "grant_type=password&username=" + url_username + "&password=" + url_password
        headers = {
            'content-type': "application/x-www-form-urlencoded",
            'authorization': "Basic " + b64bearertok,
            'cache-control': "no-cache",
        }
        try:
            r = requests.post(token_url, headers=headers, data=payload)
        except:
            # print "request timed out"
            ret['text'] = json.dumps("connection timed out: increase anchore_auth_conn_timeout higher or try again")
            return (False, ret)

        ret['text'] = r.text
        ret['status_code'] = r.status_code
        if r.status_code == 200:
            new_anchore_auth['token_info'] = json.loads(r.text)
            ret['success'] = True
        else:
            # print "unknown token get error: "
            return (False, ret)

    elif forcerefresh:
        # print "refreshening"
        payload = "grant_type=refresh_token&refresh_token=" + token_info['refreshToken']
        headers = {
            'content-type': "application/x-www-form-urlencoded",
            'authorization': "Basic " + b64bearertok,
            'cache-control': "no-cache",
        }
        try:
            r = requests.post(token_url, headers=headers, data=payload)
        except:
            # print "request timed out"
            ret['text'] = json.dumps("connection timed out: increase anchore_auth_conn_timeout higher or try again")
            return (False, ret)

        ret['text'] = r.text
        ret['status_code'] = r.status_code
        if r.status_code == 200:
            new_anchore_auth['token_info'] = json.loads(r.text)
            ret['success'] = True
        else:
            # print "refresh token invalid"
            return (False, ret)

    else:
        pass
        # print "skipping token_info get"

    if not user_info or forcerefresh:
        # Update the cached local user data
        new_user_info = get_current_user_info(anchore_auth)
        new_anchore_auth['user_info'] = new_user_info

    if anchore_auth != new_anchore_auth:
        anchore_auth.update(new_anchore_auth)
        anchore_auth_save(anchore_auth, auth_file)
    else:
        pass
        # print "skipping save"

    return (True, ret)


def anchore_auth_get(anchore_auth, url, timeout=None, retries=None):
    # make a request
    if not timeout:
        timeout = int(anchore_auth['conn_timeout'])

    if not retries:
        retries = int(anchore_auth['max_retries'])

    timeout = int(timeout)
    retries = int(retries)

    ret = {'status_code': 1, 'text': '', 'success': False}

    success = False
    count = 0
    # retries = anchore_auth['max_retries']

    while (not success and count < retries):
        count += 1
        _logger.debug("get attempt " + str(count) + " of " + str(retries))
        try:
            rc, record = anchore_auth_refresh(anchore_auth, forcerefresh=False)
            if not rc:
                # print "cannot get valid auth token"
                ret['text'] = record['text']
                return (ret)
            else:
                token_info = anchore_auth['token_info']
                accessToken = token_info['accessToken']
                headers = {"Authorization": "Bearer " + accessToken, "Cache-Control": "no-cache"}

                _logger.debug("making authenticated request to url: " + str(url))
                r = requests.get(url, headers=headers, timeout=timeout)
                _logger.debug("\tresponse status_code: " + str(r.status_code))
                if r.status_code == 401:
                    _logger.debug("\tresponse body: " + str(r.text))
                    resp = json.loads(r.text)
                    if resp['name'] == 'invalid_token':
                        # print "bad tok - attempting to refresh"
                        rc, record = anchore_auth_refresh(anchore_auth, forcerefresh=True)
                        if not rc:
                            # start over and retry
                            # print "refresh token failed, invalidating tok and starting over"
                            anchore_auth_invalidate(anchore_auth)

                elif r.status_code == 200:
                    success = True
                    ret['success'] = True
                elif r.status_code == 404:
                    success = True
                    ret['success'] = False
                elif r.status_code == 403:
                    success = True
                    ret['success'] = False
                    ret['err_msg'] = 'Access denied, check your access tier'

                ret['status_code'] = r.status_code
                ret['text'] = r.text

        except requests.exceptions.ConnectTimeout as err:
            _logger.debug("attempt failed: " + str(err))
            ret['text'] = "server error: timed_out: " + str(err)
            # return(ret)

        except Exception as err:
            _logger.debug("attempt failed: " + str(err))
            ret['text'] = "server error: " + str(err)
            # return(ret)

    return (ret)



#!/usr/bin/env python

import sys
import os
import re
import collections
import anchore.anchore_utils

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <default>| binmax0 binmax1 binmax2 ... binmaxN\nhelp: use 'default' for default bins, otherwith specify a list of integers starting with 0 to define custom bins.  ex: 0 10 100 200")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

outlist = list()
warns = list()

custombins = list()
for name in config['params']:
    if name == 'all' or name == 'default':
        break
    try:
        custombins.append(int(name))
    except:
        warns.append(["Invalid query arguments specified, using defaults.  Input must be 'default' or a list of increasing integers (bin maximums: ex '10 20 50 100 1000')."])

if not custombins:
    bins = [0, 25, 50, 100, 250, 500, 1000]
else:
    bins = custombins

result = collections.OrderedDict()
for i in range(1, len(bins)):
    bstr = str(bins[i-1]) + "-" + str(bins[i])
    result[bstr] = {'count':0, 'imageIds':list()}
bstr = str(bins[-1]) + "++"
result[bstr] = {'count':0, 'imageIds':list()}

outlist.append(["Size_Range(MBs)", "Image_Count", "Image_Ids"])

allimages = {}
for imageId in config['images']:
    try:
        image_data = anchore.anchore_utils.load_image(imageId)
        megas = int(image_data['meta']['sizebytes']) / 1000000
        match = False
        for i in range(1, len(bins)):
            if megas <= bins[i]:
                bstr = str(bins[i-1]) + "-" + str(bins[i])
                result[bstr]['count'] += 1
                result[bstr]['imageIds'].append(imageId)
                match = True
                break
                
        if not match:
            bstr = str(bins[-1]) + "++"
            result[bstr]['count'] += 1
            result[bstr]['imageIds'].append(imageId)

    except Exception as err:
        warns.append(["cannot calculate size for image: " + imageId+ ": error: " + str(err)])

for bstr in result.keys():
    if result[bstr]['imageIds']:
        #outlist.append([bstr, str(result[bstr]['count']), ','.join(result[bstr]['imageIds'])])
        outlist.append([bstr, str(result[bstr]['count']), ' '.join(result[bstr]['imageIds'])])
    else:
        outlist.append([bstr, str(result[bstr]['count']), "N/A"])

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_kvfile_fromlist(config['output_warns'], warns)

allimages.clear()
sys.exit(0)



import os
import re
import subprocess
import random
import shutil
import logging
import time

from anchore import anchore_utils
from anchore.util import scripting, contexts

class Navigator(object):
    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages):
        self.config = anchore_config
        self.allimages = allimages
        self.anchore_datadir = self.config['image_data_store']
        self.images = list()

        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=contexts['docker_cli'], anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], docker_images=contexts['docker_images'], must_be_analyzed=True, must_load_all=True)

    def add_images(self, imagelist):
        newimages = anchore_utils.image_context_add(imagelist, self.allimages, docker_cli=contexts['docker_cli'], anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], docker_images=contexts['docker_images'], must_be_analyzed=True, must_load_all=True)

        self.images = list(set(self.images) | set(newimages))

    def generate_reports(self):
        ret = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            baseId = image.get_earliest_base()
            bimage = self.allimages[baseId]
            sizebytes = image.meta['sizebytes']

            image_report = image.get_image_report()
            analysis_report = image.get_analysis_report()
            gates_report = image.get_gates_report()
            gates_eval_report = image.get_gates_eval_report()

            record = {
                'image_report': image_report,
                'analysis_report': analysis_report,
                'gates_report': gates_report,
                'gates_eval_report': gates_eval_report,
                'result': {
                    'header':['Image_Id', 'Type', 'Current_Tags', 'All_Tags', 'Gate_Status', 'Size(bytes)', 'Counts', 'Base_Diffs'],
                    'rows': list()
                }
            }

            shortId = image.meta['shortId']
            usertype = str(image.get_usertype())
            currtags = ','.join(image.get_alltags_current())
            alltags = ','.join(image.get_alltags_ever())

            gateaction = 'UNKNOWN'
            for g in gates_eval_report:
                if g['trigger'] == 'FINAL':
                    gateaction = g['action']
                    break

            try:
                pnum = str(len(anchore_utils.load_analysis_output(image.meta['imageId'], 'package_list', 'pkgs.all').keys()))
            except:
                pnum = "N/A"
            try:
                fnum = str(len(anchore_utils.load_analysis_output(image.meta['imageId'], 'file_list', 'files.all').keys()))
            except:
                fnum = "N/A"
            try:
                snum = str(len(anchore_utils.load_analysis_output(image.meta['imageId'], 'file_suids', 'files.suids').keys()))
            except:
                fnum = "N/A"

            analysis_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            compare_str = "N/A"

            if image.meta['imageId'] != baseId:
                diffdata = anchore_utils.diff_images(image.meta['imageId'], baseId)            
                record['base_compare_data'] = diffdata
                pnum = "N/A"
                if 'package_list' in diffdata and 'pkgs.all' in diffdata['package_list']:
                    for module_type in diffdata['package_list']['pkgs.all']:
                        pnum = str(len(diffdata['package_list']['pkgs.all'][module_type]))
                        break

                fnum = "N/A"
                if 'file_list' in diffdata and 'files.all' in diffdata['file_list']:
                    for module_type in diffdata['file_list']['files.all']:
                        fnum = str(len(diffdata['file_list']['files.all'][module_type]))

                snum = "N/A"
                if 'file_suids' in diffdata and 'files.suids' in diffdata['file_suids']:
                    for module_type in diffdata['file_suids']['files.suids']:
                        snum = str(len(diffdata['file_suids']['files.suids'][module_type]))

                compare_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            row = [ shortId, usertype, currtags, alltags, gateaction, sizebytes, analysis_str, compare_str ]

            record['result']['rows'].append(row)

            ret[imageId] = record
        return ret

    def get_images(self):
        return(self.images)

    def get_dockerfile_contents(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            #result[imageId] = image.get_dockerfile_contents()
            record = {'result':{}}
            record['result']['header'] = ['Image_Id', 'Mode', 'Dockerfile_Contents']
            record['result']['rows'] = list()
            (dbuf, mode) = image.get_dockerfile_contents()
            record['result']['rows'].append([image.meta['shortId'], mode, dbuf])
            result[imageId] = record
        return(result)

    def get_familytree(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['Image_Id', 'Current_Repo_Tags', 'Past_Repo_Tags', 'Image_Type']
            record['result']['rows'] = list()
            for fid in image.get_familytree():
                fimage = self.allimages[fid]
                fidstr = fimage.meta['shortId']
                curr_tags = ','.join(fimage.get_alltags_current())
                past_tags = ','.join(fimage.get_alltags_past())
                
                usertype = fimage.get_usertype()
                if usertype == "anchorebase":
                    userstr = "Anchore Base Image"
                elif usertype == "base":
                    userstr = "Base"
                elif usertype == "oldanchorebase":
                    userstr = "Previous Anchore Base Image"
                elif usertype == "user":
                    userstr = "User Image"
                else:
                    userstr = "Intermediate"

                record['result']['rows'].append([fidstr, curr_tags, past_tags, userstr])

            result[imageId] = record
            
        return(result)

    def get_layers(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['Layer_Id']
            record['result']['rows'] = list()
            for fid in image.get_layers() + [imageId]:
                record['result']['rows'].append([fid])

            result[imageId] = record
            
        return(result)

    def get_taghistory(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['Image_Id', 'Date', 'Known_Tags']
            record['result']['rows'] = list()
            for tagtup in image.get_tag_history():
                tstr = time.ctime(int(tagtup[0]))
                tagstr = ','.join(tagtup[1])
                record['result']['rows'].append([image.meta['shortId'], tstr, tagstr])

            try:
                currtags = ','.join(image.get_alltags_current())
                record['result']['rows'].append([image.meta['shortId'], time.ctime(), currtags])
            except:
                pass

            result[imageId] = record
        return(result)

    def unpack(self, destdir='/tmp'):
        ret = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            outdir = image.unpack(docleanup=False, destdir=destdir)
            if not outdir:
                self._logger.warn("failed to unpack image ("+str(imageId)+")")
            else:
                self._logger.debug("Unpacked image " + image.meta['shortId'] + " in " + outdir)
                ret[imageId] = outdir
        return(ret)

    def run(self):
        return(True)

    def find_query_command(self, action):
        cmdobj = None
        mode = None
        cmd = None
        rc = False
        try:
            path_overrides = ['/'.join([self.config['user_scripts_dir'], 'queries'])]
            if self.config['extra_scripts_dir']:
                path_overrides = path_overrides + ['/'.join([self.config['extra_scripts_dir'], 'queries'])]

            cmdobj = scripting.ScriptExecutor(path='/'.join([self.config['scripts_dir'], 'queries']), script_name=action, path_overrides=path_overrides)
            cmd = cmdobj.thecmd
            mode = 'query'
            rc = True
        except ValueError as err:
            raise err
        except Exception as err:
            errstr = str(err)
            try:
                path_overrides = ['/'.join([self.config['user_scripts_dir'], 'multi-queries'])]
                if self.config['extra_scripts_dir']:
                    path_overrides = path_overrides + ['/'.join([self.config['extra_scripts_dir'], 'multi-queries'])]
                cmdobj = scripting.ScriptExecutor(path='/'.join([self.config['scripts_dir'], 'multi-queries']), script_name=action, path_overrides=path_overrides)
                cmd = cmdobj.thecmd
                mode = 'multi-query'
                rc = True
            except ValueError as err:
                raise err
            except Exception as err:
                raise err

        ret = (rc, mode, cmdobj)
        return(ret)

    def execute_query(self, imglist, se, params):
        success = True
        datadir = self.config['image_data_store']
        outputdir = '/'.join([self.config['anchore_data_dir'], "querytmp", "query." + str(random.randint(0, 99999999))])
        if not os.path.exists(outputdir):
            os.makedirs(outputdir)

        imgfile = '/'.join([self.config['anchore_data_dir'], "querytmp", "queryimages." + str(random.randint(0, 99999999))])
        anchore_utils.write_plainfile_fromlist(imgfile, imglist)

        cmdline = ' '.join([imgfile, datadir, outputdir])
        if params:
            cmdline = cmdline + ' ' + ' '.join(params)

        meta = {}

        try:
            (cmd, rc, sout) = se.execute(capture_output=True, cmdline=cmdline)
            if rc:
                self._logger.error("Query command ran but execution failed" )
                self._logger.error("Query command: (" + ' '.join([se.thecmd, cmdline])+")")
                self._logger.error("Query output: (" + str(sout) + ")")
                self._logger.error("Exit code: (" + str(rc)+")")
                raise Exception("Query ran but exited non-zero.")
        except Exception as err:
            raise Exception("Query execution failed: " + str(err))
        else:
            try:
                #outputs = os.listdir(outputdir)
                warnfile = False
                found = False
                for f in os.listdir(outputdir):
                    if re.match(".*\.WARNS", f):
                        warnfile = '/'.join([outputdir, f])
                    else:
                        ofile = '/'.join([outputdir, f])
                        found=True

                if not found:
                    raise Exception("No output files found after executing query command\n\tCommand Output:\n"+sout+"\n\tInfo: Query command should have produced an output file in: " + outputdir)

                orows = list()

                try:
                    frows = anchore_utils.read_kvfile_tolist(ofile)
                    header = frows[0]
                    rowlen = len(header)
                    for row in frows[1:]:
                        if len(row) != rowlen:
                            raise Exception("Number of columns in data row ("+str(len(row))+") is not equal to number of columns in header ("+str(rowlen)+")\n\tHeader: "+str(header)+"\n\tOffending Row: "+str(row))
                        orows.append(row)
                except Exception as err:
                    raise err 

                if warnfile:
                    try:
                        meta['warns'] = anchore_utils.read_plainfile_tolist(warnfile)
                    except:
                        pass

                meta['queryparams'] = ','.join(params)
                meta['querycommand'] = cmd
                try:
                    i = header.index('URL')
                    meta['url_column_index'] = i
                except:
                    pass
                meta['result'] = {}
                meta['result']['header'] = header
                meta['result']['rowcount'] = len(orows)
                try:
                    #meta['result']['colcount'] = len(orows[0])
                    meta['result']['colcount'] = len(header)
                except:
                    meta['result']['colcount'] = 0
                meta['result']['rows'] = orows

            except Exception as err:
                self._logger.error("Query output handling failed: ")
                self._logger.error("\tCommand: " + str(cmd))
                self._logger.error("\tException: " + str(err))
                success = False
        finally:
            os.remove(imgfile)
            if outputdir:
                shutil.rmtree(outputdir)

        ret = [success, cmd, meta]
        return(ret)

    def list_query_commands(self, command=None):
        result = {}

        record = {'result':{}}
        record['result']['header'] = ["Query", "Help_String"]
        record['result']['rows'] = list()

        if command:
            try:
                (rc, command_type, se) = self.find_query_command(command)
                if rc:
                    (cmd, rc, sout) = se.execute(capture_output=True, cmdline='help')
                    if rc == 0:
                        record['result']['rows'].append([command, sout])
            except Exception as err:
                pass
                
        else:
            paths = list()
            paths.append('/'.join([self.config['scripts_dir'], "queries"]))
            paths.append('/'.join([self.config['scripts_dir'], "multi-queries"]))

            if self.config['user_scripts_dir']:
                paths.append('/'.join([self.config['user_scripts_dir'], 'queries']))
                paths.append('/'.join([self.config['user_scripts_dir'], 'multi-queries']))

            if self.config['extra_scripts_dir']:
                paths.append('/'.join([self.config['extra_scripts_dir'], 'queries']))
                paths.append('/'.join([self.config['extra_scripts_dir'], 'multi-queries']))

            for dd in paths:
                if not os.path.exists(dd):
                    continue
                for d in os.listdir(dd):
                    command = re.sub("(\.py|\.sh)$", "", d)
                    try:
                        (rc, command_type, se) = self.find_query_command(command)
                        if rc:
                            (cmd, rc, sout) = se.execute(capture_output=True, cmdline='help')
                            if rc == 0:
                                record['result']['rows'].append([command, sout])
                    except Exception as err:
                        pass

        result['list_query_commands'] = record
        return(result)

    def check_for_warnings(self, result):
        for k in result.keys():
            if 'warns' in result[k]:
                return(True)
        return(False)

    def run_query(self, query):
        result = {}

        if len(query) == 0:
            return(self.list_query_commands())
        elif len(query) == 1:
            action = query[0]
            params = ['help']
        else:
            action = query[0]
            params = query[1:]

        if re.match(r".*(\.\.|~|/).*", action) or re.match(r"^\..*", action):
            self._logger.error("invalid query string (bad characters in string)")
            return(False)
                    
        try:
            (rc, command_type, se) = self.find_query_command(action)
        except Exception as err:
             raise err

        if not rc:
            self._logger.error("cannot find query command in Anchore query directory: " + action + "(.py|.sh)")
            return(False)

        if params and params[0] == 'help':
            return(self.list_query_commands(action))

        outputdir = None

        if command_type == 'query':
            for imageId in self.images:
                (rc, cmd, output) = self.execute_query([imageId], se, params)
                if rc:
                    result[imageId] = output

        elif command_type == 'multi-query':
            (rc, cmd, output) = self.execute_query(self.images, se, params)
            if rc:
                result['multi'] = output

        return(result)




import copy
import subprocess

import sys
import os
import re
import json 
import random 
import shutil
import hashlib

from anchore import anchore_utils
import logging

from anchore.util import scripting, contexts

DEVNULL=open(os.devnull, 'wb')

class Controller(object):
    """
    Component that manages gate execution on images.
    """

    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages, force=False):
        self.config = anchore_config
        self.allimages = allimages
        self.force = force        
        self.anchore_datadir = self.config['image_data_store']
        
        if len(imagelist) <= 0:
            raise Exception("No images given to evaluate")

        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=contexts['docker_cli'], anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], docker_images=contexts['docker_images'], must_be_analyzed=True, must_load_all=True)

        self.anchoreDB = contexts['anchore_db']

        self.default_gatepol = '/'.join([self.config.config_dir, "anchore_gate.policy"])
        self.default_global_whitelist = os.path.join(self.config.config_dir, "anchore_global.whitelist")

        self.policy_override = None
        self.global_whitelist_override = None
        self.show_triggerIds = False

    def read_policy(self, policydata):
        policies = {}
        for l in policydata:
            l = l.strip()
            patt = re.compile('^\s*#')

            if (l and not patt.match(l)):
                polinput = l.split(':')
                module = polinput[0]
                check = polinput[1]
                action = polinput[2]
                modparams = ""
                if (len(polinput) > 3):
                    modparams = ':'.join(polinput[3:])

                if module not in policies:
                    policies[module] = {}

                if check not in policies[module]:
                    policies[module][check] = {}

                policies[module][check]['action'] = action
                policies[module][check]['params'] = modparams

        return(policies)

    def merge_policies(self, poldst, polsrc):
        polret = copy.deepcopy(poldst)
        for sk in polsrc.keys():
            if sk not in polret:
                polret[sk] = polsrc[sk]

            for smk in polsrc[sk].keys():
                if smk not in polret[sk]:
                    polret[sk] = polsrc[sk]
        
        return(polret)

    def save_policy(self, imageId, policy):
        outlist = list()

        for k in policy.keys():
            for c in policy[k].keys():
                if policy[k][c]['params']:
                    outline = k + ":" + c + ":" + policy[k][c]['action'] + ":" + policy[k][c]['params']
                else:
                    outline = k + ":" + c + ":" + policy[k][c]['action']
                outlist.append(outline)

        self.anchoreDB.save_gate_policy(imageId, outlist)
        return(True)

    def get_images(self):
        return(self.images)

    def get_image_policies(self, image):
        # load default and image override policies, merge (if new
        # checks are in default), and save (if there is a diff after
        # the merge)

        policy_data = anchore_utils.read_plainfile_tolist(self.default_gatepol)
        default_policies = self.read_policy(policy_data)

        policy_data = self.anchoreDB.load_gate_policy(image.meta['imageId'])
        image_policies = self.read_policy(policy_data)

        if image_policies and default_policies:
            policies = self.merge_policies(image_policies, default_policies)
            if policies != image_policies:
                self.save_policy(image.meta['imageId'], policies)
        else:
            policies = default_policies
            self.save_policy(image.meta['imageId'], policies)

        return(policies)

    def load_global_whitelist(self):
        ret = []
        whitelist_data = []
        whitelist_file = None

        if self.global_whitelist_override and os.path.exists(self.global_whitelist_override):
            whitelist_file = self.global_whitelist_override
        elif self.default_global_whitelist and os.path.exists(self.default_global_whitelist):
            whitelist_file = self.default_global_whitelist
        else:
            self._logger.debug("no global whitelist can be found, skipping")

        if whitelist_file:
            whitelist_data = anchore_utils.read_kvfile_tolist(whitelist_file)

        for item in whitelist_data:
            if item[0] and not re.match("^#", item[0]) and len(item) > 1:
                store = item[0:2]
                ret.append(store)

        return(ret)

    def load_whitelist(self, image):
        ret = {'ignore':[], 'enforce':[]}

        data = self.anchoreDB.load_gate_whitelist(image.meta['imageId'])
        if not data:
            return(ret)

        for l in data:
            try:
                if re.match("^#.*", l):
                    l = re.sub("^#", "", l)
                    json_dict = json.loads(l)
                    ret['ignore'].append(json_dict)
                else:
                    json_dict = json.loads(l)
                    ret['enforce'].append(json_dict)
            except:
                pass

        return(ret)

    def save_whitelist(self, image, loaded, latest):
        new = {'ignore':list(loaded['ignore']), 'enforce':loaded['enforce']}

        whitelist = self.anchoreDB.load_gate_whitelist(image.meta['imageId'])
        if not whitelist:
            outlist = list()
            for i in latest:
                outlist.append(json.dumps(i))
            self.anchoreDB.save_gate_whitelist(image.meta['imageId'], outlist)
        else:
            newpol = False
            for i in latest:
                # add evaled policy if not in the loaded whitelist
                if i not in new['ignore'] and i not in new['enforce']:
                    new['enforce'].append(i)
                    newpol = True

            # write the new whitelist, adding any new policies
            if newpol:
                outlist = list()
                for i in new['ignore']:
                    outlist.append("#"+json.dumps(i))
                for i in new['enforce']:
                    outlist.append(json.dumps(i))
                self.anchoreDB.save_gate_whitelist(image.meta['imageId'], outlist)
        return(True)

    def evaluate_gates_results(self, image):
        ret = list()
        fullret = list()
        final_gate_action = 'GO'

        policies_whitelist = self.load_whitelist(image)
        global_whitelist = self.load_global_whitelist()

        if self.policy_override:
            policy_data = anchore_utils.read_plainfile_tolist(self.policy_override)
            policies = self.read_policy(policy_data)
        else:
            policies = self.get_image_policies(image)

        for m in policies.keys():
            gdata = self.anchoreDB.load_gate_output(image.meta['imageId'], m)
            for l in gdata:
                (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
                imageId = image.meta['imageId']
                check = m
                trigger = k
                output = v
                triggerId = hashlib.md5(''.join([check,trigger,output])).hexdigest()                

                # if the output is structured (i.e. decoded as an
                # anchore compatible json string) then extract the
                # elements for display
                try:
                    json_output = json.loads(output)
                    if 'id' in json_output:
                        triggerId = str(json_output['id'])
                    if 'desc' in json_output:
                        #output = output + " description="+outputdesc
                        output = str(json_output['desc'])
                except:
                    pass
                
                if k in policies[m]:
                    trigger = k
                    action = policies[check][trigger]['action']

                    #r = {'imageId':image.meta['imageId'], 'check':m, 'trigger':k, 'output':v, 'action':policies[m][k]['action']}
                    r = {'imageId':imageId, 'check':check, 'triggerId':triggerId, 'trigger':trigger, 'output':output, 'action':action}
                    # this is where whitelist check should go
                    whitelisted = False
                    whitelist_type = "none"

                    if [m, triggerId] in global_whitelist:
                        whitelisted = True
                        whitelist_type = "global"
                    elif r in policies_whitelist['ignore']:
                        whitelisted = True
                        whitelist_type = "image"
                    
                    fullr = {}
                    fullr.update(r)
                    fullr['whitelisted'] = whitelisted
                    fullr['whitelist_type'] = whitelist_type
                    fullret.append(fullr)

                    if not whitelisted:
                        if policies[m][k]['action'] == 'STOP':
                            final_gate_action = 'STOP'
                        elif final_gate_action != 'STOP' and policies[m][k]['action'] == 'WARN':
                            final_gate_action = 'WARN'
                        ret.append(r)
                    else:
                        # whitelisted, skip evaluation
                        pass
        
        self.save_whitelist(image, policies_whitelist, ret)

        ret.append({'imageId':image.meta['imageId'], 'check':'FINAL', 'trigger':'FINAL', 'output':"", 'action':final_gate_action})
        fullret.append({'imageId':image.meta['imageId'], 'check':'FINAL', 'trigger':'FINAL', 'output':"", 'action':final_gate_action, 'whitelisted':False, 'whitelist_type':"none", 'triggerId':"N/A"})
        for i in ret:
            self.anchoreDB.del_gate_eval_output(image.meta['imageId'], i['check'])

        evals = {}
        for i in ret:
            if i['check'] not in evals:
                evals[i['check']] = list()
            evals[i['check']].append(' '.join([i['trigger'], i['action']]))

        for i in evals.keys():
            self.anchoreDB.save_gate_eval_output(image.meta['imageId'], i, evals[i])

        self.anchoreDB.save_gates_eval_report(image.meta['imageId'], ret)
        return(ret, fullret)

    def execute_gates(self, image, refresh=True):
        self._logger.debug("gate policy evaluation for image "+str(image.meta['imagename'])+": begin")
        success = True

        imagename = image.meta['imageId']
        imagedir = image.anchore_imagedir
        gatesdir = '/'.join([self.config["scripts_dir"], "gates"])
        workingdir = '/'.join([self.config['anchore_data_dir'], 'querytmp'])
        outputdir = workingdir
        
        if not self.force and os.path.exists(imagedir + "/gates.done"):
            self._logger.info(image.meta['shortId'] + ": evaluated.")
            return(True)

        self._logger.info(image.meta['shortId'] + ": evaluating policies ...")
        
        for d in [outputdir, workingdir]:
            if not os.path.exists(d):
                os.makedirs(d)

        imgfile = '/'.join([workingdir, "queryimages." + str(random.randint(0, 99999999))])
        anchore_utils.write_plainfile_fromstr(imgfile, image.meta['imageId'])

        if self.policy_override:
            policy_data = anchore_utils.read_plainfile_tolist(self.policy_override)
            policies = self.read_policy(policy_data)
        else:
            policies = self.get_image_policies(image)

        paramlist = list()
        for p in policies.keys():
            for t in policies[p].keys():
                if 'params' in policies[p][t] and policies[p][t]['params']:
                    paramlist.append(policies[p][t]['params'])
        if len(paramlist) <= 0:
            paramlist.append('all')

        path_overrides = ['/'.join([self.config['user_scripts_dir'], 'gates'])]
        if self.config['extra_scripts_dir']:
            path_overrides = path_overrides + ['/'.join([self.config['extra_scripts_dir'], 'gates'])]

        results = scripting.ScriptSetExecutor(path=gatesdir, path_overrides=path_overrides).execute(capture_output=True, fail_fast=True, cmdline=' '.join([imgfile, self.config['image_data_store'], outputdir, ' '.join(paramlist)]))

        os.remove(imgfile)

        for r in results:
            (cmd, retcode, output) = r
            if retcode:
                self._logger.error("FAILED")
                self._logger.error("\tCMD: " + cmd)
                self._logger.error("\tEXITCODE: " + str(retcode))
                self._logger.error("\tOUTPUT: " + output)
                success = False
            else:
                self._logger.debug("")
                self._logger.debug("\tCMD: " + cmd)
                self._logger.debug("\tEXITCODE: " + str(retcode))
                self._logger.debug("\tOUTPUT: " + output)
                self._logger.debug("")

        if success:
            report = self.generate_gates_report(image)
            self.anchoreDB.save_gates_report(image.meta['imageId'], report)
            self._logger.info(image.meta['shortId'] + ": evaluated.")

        self._logger.debug("gate policy evaluation for image "+str(image.meta['imagename'])+": end")
        return(success)

    def generate_gates_report(self, image):
        # this routine reads the results of image gates and generates a formatted report
        report = {}

        outputs = self.anchoreDB.list_gate_outputs(image.meta['imageId'])
        for d in outputs:
            report[d] = self.anchoreDB.load_gate_output(image.meta['imageId'], d)

        return(report)

    def result_get_highest_action(self, results):
        highest_action = 0
        for k in results.keys():
            action = results[k]['result']['final_action']
            if action == 'STOP':
                highest_action = 1
            elif highest_action == 0 and action == 'WARN':
                highest_action = 2
            
        return(highest_action)

    def run_gates(self, policy=None, refresh=True, global_whitelist=None, show_triggerIds=False, show_whitelisted=False):
        # actually run the gates
        ret = {}

        if policy:
            self.policy_override = policy

        if global_whitelist:
            self.global_whitelist_override = global_whitelist

        self.show_triggerIds = show_triggerIds
        self.show_whitelisted = show_whitelisted

        for imageId in self.images:
            image = self.allimages[imageId]

            if not self.execute_gates(image, refresh=refresh):
                raise Exception("One or more gates failed to execute")

            results, fullresults = self.evaluate_gates_results(image)

            record = {}
            record['result'] = {}

            #record['result']['header'] = ['Image_Id', 'Repo_Tag', 'Gate', 'Trigger', 'Check_Output', 'Gate_Action']        
            record['result']['header'] = ['Image_Id', 'Repo_Tag']
            if self.show_triggerIds:
                record['result']['header'].append('Trigger_Id')
            record['result']['header'] += ['Gate', 'Trigger', 'Check_Output', 'Gate_Action']
            if self.show_whitelisted:
                record['result']['header'].append('Whitelisted')

            record['result']['rows'] = list()

            for m in fullresults:
                id = image.meta['imageId']
                name = image.get_human_name()
                gate = m['check']
                trigger = m['trigger']
                output = m['output']
                triggerId = m['triggerId']
                action = m['action']

                row = [id[0:12], name]
                if self.show_triggerIds:
                    row.append(triggerId)
                row += [gate, trigger, output, action]
                if self.show_whitelisted:
                    row.append(m['whitelist_type'])

                if not m['whitelisted'] or show_whitelisted:
                    record['result']['rows'].append(row)

                if gate == 'FINAL':
                    record['result']['final_action'] = action

            ret[imageId] = record
        return(ret)

    def editpolicy(self):
        return(self.edit_policy_file(editpolicy=True))

    def editwhitelist(self):
        return(self.edit_policy_file(whitelist=True))

    def listpolicy(self):
        ret = {}
        for imageId in self.images:
            if imageId in self.allimages:
                image = self.allimages[imageId]
                image_pol = self.get_image_policies(image)
                ret[imageId] = image_pol
        return(ret)

    def rmpolicy(self):
        for imageId in self.images:
            if imageId in self.allimages:
                self.anchoreDB.del_gate_policy(imageId)

        return(True)

    def updatepolicy(self, newpolicyfile):
        policy_data = anchore_utils.read_plainfile_tolist(newpolicyfile)
        newpol = self.read_policy(policy_data)
        for imageId in self.images:
            if imageId in self.allimages:
                try:
                    self.save_policy(imageId, newpol)
                except Exception as err:
                    self._logger.error("failed to update policy for image ("+imageId+"). bailing out: " + str(err))
                    return(False)
        return(True)

    def edit_policy_file(self, editpolicy=False, whitelist=False):
        ret = True

        if not editpolicy and not whitelist:
            # nothing to do
            return(ret)

        for imageId in self.images:
            if editpolicy:
                data = self.anchoreDB.load_gate_policy(imageId)
            else:
                data = self.anchoreDB.load_gate_whitelist(imageId)

            if not data:
                self._logger.info("Cannot find existing data to edit, skipping: " + str(imageId))
            else:
                tmpdir = anchore_utils.make_anchoretmpdir("/tmp")
                try:
                    thefile = os.path.join(tmpdir, "anchorepol."+imageId)
                    anchore_utils.write_plainfile_fromlist(thefile, data)
                    if "EDITOR" in os.environ:
                        cmd = os.environ["EDITOR"].split()
                        cmd.append(thefile)
                        try:
                            subprocess.check_output(cmd, shell=False)
                        except:
                            ret = False
                    elif os.path.exists("/bin/vi"):
                        try:
                            rc = os.system("/bin/vi " + thefile)
                            if rc:
                                ret = False
                        except:
                            ret = False
                    else:
                        self._logger.info("Cannot find editor to use: please set the EDITOR environment variable and try again")
                        break
                        ret = False

                    newdata = anchore_utils.read_plainfile_tolist(thefile)

                    if editpolicy:
                        self.anchoreDB.save_gate_policy(imageId, newdata)
                    else:
                        self.anchoreDB.save_gate_whitelist(imageId, newdata)
                except Exception as err:
                    pass
                finally:
                    if tmpdir:
                        shutil.rmtree(tmpdir)

        return(ret)



#!/usr/bin/env python

import sys
import os
import json
import re
import anchore.anchore_utils

gate_name = "DOCKERFILECHECK"
triggers = {
    'EXPOSE':
    {
        'description':'triggers if Dockerfile is EXPOSEing ports that are not in ALLOWEDPORTS, or are in DENIEDPORTS',
        'params':'ALLOWEDPORTS,DENIEDPORTS'
    },
    'NOFROM':
    {
        'description':'triggers if there is no FROM line specified in the Dockerfile',
        'params':'None'
    },
    'FROMSCRATCH':
    {
        'description':'triggers the FROM line specified "scratch" as the parent',
        'params':'None'
    },
    'NOTAG':
    {
        'description':'triggers if the FROM container specifies a repo but no explicit, non-latest tag ',
        'params':'None'
    },
    'SUDO':
    {
        'description':'triggers if the Dockerfile contains operations running with sudo',
        'params':'None'
    },
    'VOLUMEPRESENT':
    {
        'description':'triggers if the Dockerfile contains a VOLUME line',
        'params':'None'
    }
}

try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, gate_name, gate_help=triggers)
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    print "ERROR: could not set up environment for gate"
    sys.exit(1)

imgid = config['imgid']

outlist = list()

try:
    params = config['params']
except:
    params = None

parsed_params = {}
if params:
    for paramstr in params:
        try:
            key, val = paramstr.split("=")
            parsed_params[key] = list()
            for p in val.split(","):
                parsed_params[key].append(p)
        except:
            pass

# do something
try:
    ireport = anchore.anchore_utils.load_image_report(imgid)
    if 'dockerfile_mode' in ireport and ireport['dockerfile_mode'] == "Actual":
        if 'dockerfile_contents' in ireport:
            dockerfile_contents = ireport['dockerfile_contents']
            fromstr = None
            exposestr = None
            volumestr = None
            sudostr = None

            for line in dockerfile_contents.splitlines():
                line = line.strip()
                if re.match("^\s*(FROM|"+'FROM'.lower()+")\s+(.*)", line):
                    fromstr = re.match("^\s*(FROM|"+'FROM'.lower()+")\s+(.*)", line).group(2)
                elif re.match("^\s*(EXPOSE|"+'EXPOSE'.lower()+")\s+(.*)", line):
                    exposestr = re.match("^\s*(EXPOSE|"+'EXPOSE'.lower()+")\s+(.*)", line).group(2)
                elif re.match("^\s*(VOLUME|"+'VOLUME'.lower()+")\s+(.*)", line):
                    volumestr = str(line)
                elif re.match(".*sudo.*", line):
                    sudostr = str(line)

            if fromstr:
                if fromstr == 'SCRATCH' or fromstr.lower() == 'scratch':
                    outlist.append("FROMSCRATCH 'FROM' container is 'scratch' - ("+str(fromstr)+")")
                elif re.match("(\S+):(\S+)", fromstr):
                    repo, tag = re.match("(\S+):(\S+)", fromstr).group(1,2)
                    if tag == 'latest':
                        outlist.append("NOTAG 'FROM' container does not specify a non-latest container tag - ("+str(fromstr)+")")
                else:
                    outlist.append("NOTAG 'FROM' container does not specify a non-latest container tag - ("+str(fromstr)+")")
            else:
                outlist.append("NOFROM No 'FROM' directive in Dockerfile")

            if exposestr:
                iexpose = exposestr.split()
                if 'DENIEDPORTS' in parsed_params:
                    if 'ALL' in parsed_params['DENIEDPORTS'] and len(iexpose) > 0:
                        outlist.append("EXPOSE Dockerfile exposes network ports but policy sets DENIEDPORTS=ALL: " + str(iexpose))
                    else:
                        for p in parsed_params['DENIEDPORTS']:
                            if p in iexpose:
                                outlist.append("EXPOSE Dockerfile exposes port ("+p+") which is in policy file DENIEDPORTS list")
                            elif p+'/tcp' in iexpose:
                                outlist.append("EXPOSE Dockerfile exposes port ("+p+"/tcp) which is in policy file DENIEDPORTS list")
                            elif p+'/udp' in iexpose:
                                outlist.append("EXPOSE Dockerfile exposes port ("+p+"/udp) which is in policy file DENIEDPORTS list")
    
                if 'ALLOWEDPORTS' in parsed_params:
                    if 'NONE' in parsed_params['ALLOWEDPORTS'] and len(iexpose) > 0:
                        outlist.append("EXPOSE Dockerfile exposes network ports but policy sets ALLOWEDPORTS=NONE: " + str(iexpose))
                    else:
                        for p in parsed_params['ALLOWEDPORTS']:
                            done=False
                            while not done:
                                try:
                                    iexpose.remove(p)
                                    done=False
                                except:
                                    done=True

                                try:
                                    iexpose.remove(p+'/tcp')
                                    done=False
                                except:
                                    done=True

                                try:
                                    iexpose.remove(p+'/udp')
                                    done=False
                                except:
                                    done=True

                        for ip in iexpose:
                            outlist.append("EXPOSE Dockerfile exposes port ("+ip+") which is not in policy file ALLOWEDPORTS list")

            if volumestr:
                outlist.append("VOLUMEPRESENT Dockerfile contains a VOLUME line: " + str(volumestr))

            if sudostr:
                outlist.append("SUDO Dockerfile contains a 'sudo' command: " + str(sudostr))

except Exception as err:
    outlist.append(gate_name + " gate failed to run with exception: " + str(err))
    #exit(1)

# write output
anchore.anchore_utils.save_gate_output(imgid, gate_name, outlist)

sys.exit(0)



import os
import sys
import shutil
import requests
import re
import json
import time
import logging

import anchore.anchore_auth
from anchore.util import contexts

# network operations
_logger = logging.getLogger(__name__)


def get_feed_list():
    ret = {'success': False, 'status_code': 1, 'text': ""}

    feedurl = contexts['anchore_config']['feeds_url']
    feed_timeout = contexts['anchore_config']['feeds_conn_timeout']
    feed_maxretries = contexts['anchore_config']['feeds_max_retries']

    baseurl = feedurl
    url = baseurl

    retlist = list()

    done = False
    while not done:
        record = anchore.anchore_auth.anchore_auth_get(contexts['anchore_auth'], url, timeout=feed_timeout,
                                                       retries=feed_maxretries)
        ret.update(record)
        if record['success']:
            data = json.loads(record['text'])
            if data and 'feeds' in data:
                retlist = retlist + data['feeds']

                if 'next_token' in data and data['next_token']:
                    url = baseurl + "?next_token=" + data['next_token']
                else:
                    done = True
            else:
                done = True
        else:
            done = True

    return (retlist, ret)


def get_group_list(feed):
    ret = {'success': False, 'status_code': 1, 'text': ""}

    feedurl = contexts['anchore_config']['feeds_url']
    feed_timeout = contexts['anchore_config']['feeds_conn_timeout']
    feed_maxretries = contexts['anchore_config']['feeds_max_retries']

    baseurl = '/'.join([feedurl, feed])
    url = baseurl

    retlist = list()

    done = False
    while not done:
        record = anchore.anchore_auth.anchore_auth_get(contexts['anchore_auth'], url, timeout=feed_timeout,
                                                       retries=feed_maxretries)
        ret.update(record)
        if record['success']:
            data = json.loads(record['text'])
            retlist = retlist + data['groups']
            if 'next_token' in data and data['next_token']:
                url = baseurl + "?next_token=" + data['next_token']
            else:
                done = True
        else:
            done = True
    return (retlist, record)


def get_group_data(feed, group, since="1970-01-01"):
    feedurl = contexts['anchore_config']['feeds_url']
    feed_timeout = contexts['anchore_config']['feeds_conn_timeout']
    feed_maxretries = contexts['anchore_config']['feeds_max_retries']

    baseurl = '/'.join([feedurl, feed, group, "?since=" + since])
    url = baseurl

    updatetime = int(time.time())

    ret = list()
    last_token = ""
    success = True
    done = False
    while not done:
        _logger.debug("data group url: " + str(url))
        record = anchore.anchore_auth.anchore_auth_get(contexts['anchore_auth'], url, timeout=feed_timeout,
                                                       retries=feed_maxretries)
        if record['success']:
            data = json.loads(record['text'])
            if 'data' in data:
                ret = ret + data['data']
                if 'next_token' in data and data['next_token']:
                    url = baseurl + "&next_token=" + data['next_token']
                    if last_token == data['next_token']:
                        done = True
                    last_token = data['next_token']
                else:
                    done = True
            else:
                success = False
                done = True
        else:
            success = False
            if record and 'err_msg' in record:
                ret = record.get('err_msg')
            done = True
    return (success, ret)


def create_feed(feed):
    if not feed:
        return (False)

    return (contexts['anchore_db'].create_feed(feed))


def create_feedgroup(feed, group):
    if not feed or not group:
        return (False)

    return (contexts['anchore_db'].create_feedgroup(feed, group))


def sync_feedmeta(default_sublist=['vulnerabilities']):
    ret = {'success': False, 'text': "", 'status_code': 1}

    try:
        feedmeta = load_anchore_feedmeta()

        feeds, record = get_feed_list()
        if feeds:
            for feedrecord in feeds:
                feed = feedrecord['name']
                if feed not in feedmeta:
                    feedmeta[feed] = {}
                feedmeta[feed].update(feedrecord)

                if 'groups' not in feedmeta[feed]:
                    feedmeta[feed]['groups'] = {}

                if 'subscribed' not in feedmeta[feed]:
                    if feed in default_sublist:
                        feedmeta[feed]['subscribed'] = True
                    else:
                        feedmeta[feed]['subscribed'] = False

                rc = create_feed(feed)

                groups, record = get_group_list(feed)
                if groups:
                    for grouprecord in groups:
                        group = grouprecord['name']
                        if group not in feedmeta[feed]['groups']:
                            feedmeta[feed]['groups'][group] = {}

                        feedmeta[feed]['groups'][group].update(grouprecord)

                        rc = create_feedgroup(feed, group)
                else:
                    ret['text'] = "WARN: cannot get list of groups from feed: " + str(feed)

            ret['success'] = True
            ret['status_code'] = 0

        else:
            ret['text'] = "cannot get list of feeds from service\nMessage from server: " + record['text']
            return (False, ret)

        save_anchore_feedmeta(feedmeta)

    except Exception as err:
        import traceback
        traceback.print_exc()
        _logger.debug("exception: " + str(err))
        ret['text'] = "exception: " + str(err)
        return (False, ret)

    return (True, ret)


def feed_group_data_exists(feed, group, datafile):
    feedmeta = load_anchore_feedmeta()
    if feed in feedmeta.keys() and group in feedmeta[feed]['groups'].keys() and 'datafiles' in feedmeta[feed]['groups'][
        group] and datafile in feedmeta[feed]['groups'][group]['datafiles']:
        return (True)
    return (False)


def sync_feeds(force_since=None, do_combine=False):
    ret = {'success': False, 'text': "", 'status_code': 1}

    feedmeta = load_anchore_feedmeta()

    feedurl = contexts['anchore_config']['feeds_url']
    try:
        for feed in feedmeta.keys():
            if feedmeta[feed]['subscribed']:
                _logger.info("syncing data for subscribed feed (" + str(feed) + ") ...")
                groups = feedmeta[feed]['groups'].keys()
                if groups:
                    for group in groups:
                        sincets = 0
                        group_meta = {}
                        if group in feedmeta[feed]['groups']:
                            group_meta = feedmeta[feed]['groups'][group]
                            if 'last_update' in group_meta:
                                sincets = group_meta['last_update']

                        if force_since:
                            sincets = float(force_since)

                        if 'datafiles' not in group_meta:
                            group_meta['datafiles'] = list()

                        updatetime = int(time.time())

                        since = time.strftime("%Y-%m-%d", time.gmtime(sincets))
                        now = time.strftime("%Y-%m-%d", time.gmtime(updatetime))

                        datafilename = "data_" + since + "_to_" + now + ".json"
                        if feed_group_data_exists(feed, group, datafilename):
                            _logger.info("\tskipping group data: " + str(group) + ": already synced")
                        else:
                            _logger.info("\tsyncing group data: " + str(group) + ": ...")
                            success, data = get_group_data(feed, group, since=since)
                            if success:
                                rc = save_anchore_feed_group_data(feed, group, datafilename, data)

                                group_meta['prev_update'] = sincets
                                group_meta['last_update'] = updatetime
                                if datafilename not in group_meta['datafiles']:
                                    group_meta['datafiles'].append(datafilename)

                                # finally, do any post download feed/group handler actions
                                rc, msg = handle_anchore_feed_post(feed, group)
                            else:
                                if data and isinstance(data, str):
                                    err_msg = data
                                else:
                                    err_msg = 'check --debug output and/or try again'

                                _logger.warn("\t\tWARN: failed to download feed/group data (" + str(feed) + "/" + str(
                                    group) + "): {}".format(err_msg))

            else:
                _logger.info("skipping data sync for unsubscribed feed (" + str(feed) + ") ...")

        ret['status_code'] = 0
        ret['success'] = True
    except Exception as err:
        import traceback
        traceback.print_exc()
        _logger.debug("exception: " + str(err))
        ret['text'] = "ERROR: " + str(err)
        return (False, ret)

    if not save_anchore_feedmeta(feedmeta):
        ret['text'] = "\t\tWARN: failed to store metadata on synced feed data"

    # if user has asked for data compress, do it now
    if do_combine:
        handle_datafile_combine()

    return (True, ret)


def check():
    if not load_anchore_feedmeta():
        return (False, "feeds are not initialized: please run 'anchore feeds sync' and try again")

    if not load_anchore_feeds_list():
        return (False, "feeds list is empty: please run 'anchore feeds sync' and try again")

    return (True, "success")


def load_anchore_feeds_list():
    ret = []
    feedmeta = load_anchore_feedmeta()
    if feedmeta:
        ret = feedmeta.values()

    return (ret)


def load_anchore_feed_groups_list(feed):
    ret = []
    feedmeta = load_anchore_feedmeta()
    if feedmeta:
        if feed in feedmeta.keys():
            ret = feedmeta[feed]['groups'].values()

    return (ret)


def load_anchore_feed_group_datameta(feed, group):
    ret = {}
    feedmeta = load_anchore_feedmeta()
    if feedmeta:
        if feed in feedmeta.keys() and group in feedmeta[feed]['groups'].keys():
            ret = feedmeta[feed]['groups'][group]

    return (ret)


def load_anchore_feedmeta():
    return (contexts['anchore_db'].load_feedmeta())


def save_anchore_feedmeta(feedmeta):
    return (contexts['anchore_db'].save_feedmeta(feedmeta))


def subscribe_anchore_feed(feed, user_tier=0):
    success = True
    msg = str(feed) + ": subscribed."

    feedmeta = load_anchore_feedmeta()
    if feed in feedmeta:
        if user_tier >= int(feedmeta[feed]['access_tier']):
            if not feedmeta[feed]['subscribed']:
                feedmeta[feed]['subscribed'] = True
                if not save_anchore_feedmeta(feedmeta):
                    msg = str(feed) + ": failed to subscribe to feed (check debug output)."
                    success = False
        else:
            msg = 'Current user does not have sufficient access tier to subscribe to feed {0}. Current tier is {1}, must be {2} to access feed'.format(feed, user_tier, feedmeta[feed]['access_tier'])
            success = False
    else:
        msg = "cannot find specified feed (" + str(feed) + "): please review the feeds list and try again"
        success = False
    return (success, msg)


def unsubscribe_anchore_feed(feed):
    success = True
    msg = str(feed) + ": unsubscribed."

    feedmeta = load_anchore_feedmeta()
    if feed in feedmeta:
        if feedmeta[feed]['subscribed']:
            feedmeta[feed]['subscribed'] = False
            if not save_anchore_feedmeta(feedmeta):
                msg = str(feed) + ": failed to unsubscribe to feed (check debug output)."
                success = False
    else:
        msg = "cannot find specified feed (" + str(feed) + "): please review the feeds list and try again"
        success = False

    return (success, msg)


def save_anchore_feed_group_data(feed, group, datafile, data):
    return (contexts['anchore_db'].save_feed_group_data(feed, group, datafile, data))


def load_anchore_feed_group_data(feed, group, datafile):
    return (contexts['anchore_db'].load_feed_group_data(feed, group, datafile))

def delete_anchore_feed_group_data(feed, group, datafile):
    return (contexts['anchore_db'].delete_feed_group_data(feed, group, datafile))

def load_anchore_feed(feed, group, ensure_unique=False):
    ret = {'success': False, 'msg': "", 'data': list()}
    datameta = {}
    doupdate = False
    feedmeta = load_anchore_feedmeta()
    if not feedmeta:
        ret['msg'] = "feed data does not exist: please sync feed data"
    elif feed in feedmeta and not feedmeta[feed]['subscribed']:
        ret['msg'] = "not currently subscribed to feed (" + str(feed) + "): please subscribe, sync, and try again"
    else:
        if feed in feedmeta and group in feedmeta[feed]['groups']:
            datameta = feedmeta[feed]['groups'][group]

        if datameta and 'datafiles' in datameta:
            unique_hash = {}
            revfiles = sorted(datameta['datafiles'])
            revfiles.reverse()
            #for datafile in sorted(datameta['datafiles']):
            for datafile in revfiles:
                thelist = load_anchore_feed_group_data(feed, group, datafile)
                if ensure_unique:
                    for el in thelist:
                        if isinstance(el, dict) and len(el.keys()) == 1:
                            elkey = el.keys()[0]
                            if elkey in unique_hash:
                                _logger.debug("FOUND duplicate entry during scan for unique data values: " + str(elkey))
                            else:
                                unique_hash[elkey] = el

                ret['data'] = ret['data'] + thelist
                ret['success'] = True
                ret['msg'] = "success"

            if ret['success'] and ensure_unique:
                ret['data'] = unique_hash.values()

        else:
            ret['msg'] = "no data exists for given feed/group (" + str(feed) + "/" + str(group) + ")"

    return (ret)


def delete_anchore_feed(feed):
    feedmeta = load_anchore_feedmeta()
    if feed in feedmeta.keys():
        if not feedmeta[feed]['subscribed']:
            try:
                for group in feedmeta[feed]['groups'].keys():
                    feedmeta[feed]['groups'][group].pop('datafiles', [])
                    feedmeta[feed]['groups'][group].pop('last_update', 0)
                    feedmeta[feed]['groups'][group].pop('prev_update', 0)

                save_anchore_feedmeta(feedmeta)
                contexts['anchore_db'].delete_feed(feed)
            except Exception as err:
                _logger.warn("could not complete delete of feed - message from service: " + str(err))

            return (True)
        else:
            _logger.warn(
                "skipping delete of feed that is marked as subscribed - please unsubscribe first and then retry the delete")

    return (True)


# TODO wip
def handle_anchore_feed_pre(feed):
    ret = True
    msg = ""
    feedmeta = load_anchore_feedmeta()
    if feed in feedmeta:
        if feed == 'vulnerabilities':
            if not feedmeta[feed]['subscribed']:
                rc, msg = subscribe_anchore_feed(feed)
                ret = rc
    return (ret, msg)


def handle_anchore_feed_post(feed, group):
    ret = True
    msg = ""
    if feed == 'imagedata':
        # handler
        d = load_anchore_feed(feed, group)
        if d and d['success']:
            for record in d['data']:
                if 'redirecturl' in record.keys():
                    for tag in record['redirecturl'].keys():
                        url = record['redirecturl'][tag]
                        imageId = tag
                        if not contexts['anchore_db'].is_image_present(imageId):
                            try:
                                r = requests.get(url, timeout=10)
                                if r.status_code == 200:
                                    data = json.loads(r.text)
                                    for imagedata in data:
                                        imagerecord = imagedata['image']['imagedata']
                                        _logger.info("\t\tpopulating anchore DB with image data: " + imageId)
                                        contexts['anchore_db'].save_image_new(imageId, report=imagerecord)
                            except Exception as err:
                                _logger.error("exception: " + str(err))
                                ret = False
                                msg = "failed to download/import image: " + imageId
                        else:
                            _logger.info("\t\tskipping: " + str(imageId) + ": already in DB")
    else:
        # no handler
        pass

    return (ret, msg)

def handle_datafile_combine():
    ret = True

    feedmeta = load_anchore_feedmeta()

    for feed in feedmeta.keys():
        if 'groups' in feedmeta[feed] and 'subscribed' in feedmeta[feed] and feedmeta[feed]['subscribed']:
            _logger.info("combining data for feed ("+str(feed)+") ...")
            for group in feedmeta[feed]['groups']:
                rawdata = load_anchore_feed(feed, group, ensure_unique=False)
                data = rawdata['data']
                uniqhash = {}
                uniq = list()
                collisions = 0
                for v in data:
                    vid = None
                    try:
                        if feed == 'vulnerabilities':
                            vid = v['Vulnerability']['Name']
                        elif feed == 'packages':
                            vid = v.keys()[0]
                    except:
                        vid = None
                        pass

                    if vid:
                        if vid not in uniqhash:
                            uniqhash[vid] = True
                            uniq.append(v)
                        else:
                            collisions = collisions + 1

                rawdata.clear()
                _logger.info("\tprocessing group data: " + str(group) + ": removed " + str(collisions) + " records as duplicate or out-of-date")

                # datafile updates
                updatetime = int(time.time())
                now = time.strftime("%Y-%m-%d", time.gmtime(updatetime))

                datafilename = "data_" + now + "_to_" + now + ".json"
                rc = save_anchore_feed_group_data(feed, group, datafilename, uniq)
                if rc:
                    if 'datafiles' in feedmeta[feed]['groups'][group]:
                        for dfile in feedmeta[feed]['groups'][group]['datafiles']:
                            if dfile != datafilename:
                                delete_anchore_feed_group_data(feed, group, dfile)

                    feedmeta[feed]['groups'][group]['datafiles'] = [datafilename]
                    save_anchore_feedmeta(feedmeta)

    return(ret)



import re
import os
import sys
import subprocess
import shutil
import tempfile
import hashlib
import time
import logging

import controller
from anchore import anchore_utils
from anchore.util import scripting
from anchore.util import contexts

class Analyzer(object):
    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages, force, args=None):
        self._logger.debug("analyzer initialization: begin")

        self.config = anchore_config
        self.allimages = allimages
        self.force = force
        self.anchore_datadir = self.config['image_data_store']

        self.dockerfile = None
        try:
            self.dockerfile = args['dockerfile']
        except:
            pass

        self.skipgates = False
        try:
            self.skipgates = args['skipgates']
        except:
            pass

        try:
            if 'isbase' in args and args['isbase']:
                usertype = 'base'
            elif 'anchorebase' in args and args['anchorebase']:
                usertype = 'anchorebase'
            else:
                usertype = None
        except:
            usertype = None

        self._logger.debug("init input processed, loading input images: " + str(imagelist))
        
        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=contexts['docker_cli'], dockerfile=self.dockerfile, anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], docker_images=contexts['docker_images'], usertype=usertype, must_load_all=True)

        self._logger.debug("loaded input images, checking that all input images have been loaded " + str(self.images))

        self.anchoreDB = contexts['anchore_db']

        self._logger.debug("analyzer initialization: end")

    def get_images(self):
        return(self.images)

    def script_is_runnable(self, script):
        suffix_list = ['.py', '.sh']
        match = False
        for s in suffix_list:
            if re.match(".*"+re.escape(s)+"$", script):
                match = True
                break
        if match and os.access(script, os.R_OK ^ os.X_OK):
            return(True)
        return(False)

    def list_analyzers(self):
        analyzerdir = '/'.join([self.config["scripts_dir"], "analyzers"])
        overrides = ['extra_scripts_dir', 'user_scripts_dir']

        scripts = {'base':list()}
        for override in overrides:
            scripts[override] = list()

        if not os.path.exists(analyzerdir):
            raise Exception("No base analyzers found - please check anchore insallation for completeness")
        else:
            for f in os.listdir(analyzerdir):
                script = os.path.join(analyzerdir, f)
                # check the script to make sure its ready to run
                if self.script_is_runnable(script):
                    scripts['base'].append(script)

        for override in overrides:
            scripts[override] = list()
            if self.config[override]:
                opath = os.path.join(self.config[override], 'analyzers')
                if os.path.exists(opath):
                    for f in os.listdir(opath):
                        script = os.path.join(opath, f)
                        if self.script_is_runnable(script):
                            scripts[override].append(script)
        return(scripts)

    def run_analyzers(self, image):
        success = True
        analyzers = self.list_analyzers()
        imagename = image.meta['imagename']
        outputdir = image.anchore_imagedir
        shortid = image.meta['shortId']
        imagedir = None

        analyzer_status = self.anchoreDB.load_analyzer_manifest(image.meta['imageId'])
        
        analyzer_config = {}
        analyzer_config_csum = None
        try:
            analyzer_config, analyzer_config_csum = anchore_utils.load_analyzer_config(self.config.config_dir)
        except:
            pass

        if 'analyzer_config_csum' in analyzer_status:
            try:
                if analyzer_status['analyzer_config_csum']['csum'] != analyzer_config_csum:
                    self._logger.debug("anchore analyzer config has been updating, forcing re-analysis")
                    self.force = True
                    analyzer_status['analyzer_config_csum']['csum'] = analyzer_config_csum
            except:
                pass
        else:
            script = 'analyzer_config_csum'
            analyzer_status[script] = {}
            analyzer_status[script]['command'] = "ANALYZER_CONFIG_META"
            analyzer_status[script]['returncode'] = 0
            analyzer_status[script]['output'] = ""
            analyzer_status[script]['outputdir'] = ""
            analyzer_status[script]['atype'] = 'base'
            analyzer_status[script]['csum'] = analyzer_config_csum
            analyzer_status[script]['timestamp'] = time.time()
            analyzer_status[script]['status'] = 'SUCCESS'
            

        results = {}
        outputdirs = {}
        torun = list()
        skip = False
        atypes = ['user_scripts_dir', 'extra_scripts_dir', 'base']

        for atype in atypes:
            for script in analyzers[atype]:
                try:
                    with open(script, 'r') as FH:
                        csum = hashlib.md5(FH.read()).hexdigest()
                except:
                    csum = "N/A"

                # decide whether or not to run the analyzer
                dorun = True
                if self.force:
                    dorun = True
                elif script in analyzer_status:
                    if csum == analyzer_status[script]['csum'] and analyzer_status[script]['returncode'] == 0:
                        dorun = False

                outputdir = cmdstr = outstr = ""
                if dorun:
                    if not skip:
                        if not imagedir:
                            self._logger.info(image.meta['shortId'] + ": analyzing ...")                            
                            imagedir = image.unpack()
                            if not imagedir:
                                self._logger.error("could not unpack image")
                                return(False)
                            
                        outputdir = tempfile.mkdtemp(dir=imagedir)
                        cmdline = ' '.join([imagename, self.config['image_data_store'], outputdir, imagedir])
                        cmdstr = script + " " + cmdline
                        cmd = cmdstr.split()
                        try:
                            self._logger.debug("running analyzer: " + cmdstr)
                            timer = time.time()
                            outstr = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
                            self._logger.debug("analyzer time (seconds): " + str(time.time() - timer))
                            rc = 0
                            self._logger.debug("analyzer status: success")
                            self._logger.debug("analyzer exitcode: " + str(rc))
                            self._logger.debug("analyzer output: " + outstr)
                        except subprocess.CalledProcessError as err:
                            rc = err.returncode
                            outstr = err.output
                        outstr = outstr.decode('utf8')
                        if rc:
                            status = 'FAILED'
                            skip = True
                            success = False
                            self._logger.error("analyzer status: failed")
                            self._logger.error("analyzer exitcode: " + str(rc))
                            self._logger.error("analyzer output: " + outstr)
                        else:
                            status = 'SUCCESS'
                    else:
                        # this means that a prior analyzer failed, so we skip the rest
                        self._logger.debug("skipping analyzer (due to prior analyzer failure): " + script)
                        outstr = ""
                        rc = 1
                        status = 'SKIPPED'

                    mtype = "base"
                    if atype == 'user_scripts_dir':
                        mtype = 'user'
                    elif atype == 'extra_scripts_dir':
                        mtype = 'extra'

                    results[script] = {}
                    results[script]['command'] = cmdstr
                    results[script]['returncode'] = rc
                    results[script]['output'] = outstr
                    results[script]['outputdir'] = outputdir
                    results[script]['atype'] = atype
                    results[script]['csum'] = csum
                    results[script]['timestamp'] = time.time()
                    results[script]['status'] = status

                    if os.path.exists(os.path.join(outputdir, 'analyzer_output')):
                        for d in os.listdir(os.path.join(outputdir, 'analyzer_output')):
                            if os.path.exists(os.path.join(outputdir, 'analyzer_output', d)):
                                for dd in os.listdir(os.path.join(outputdir, 'analyzer_output', d)):
                                    module_name = d
                                    module_value = dd
                                    if 'analyzer_outputs' not in results[script]:
                                        #results[script]['analyzer_outputs'] = {}
                                        results[script]['analyzer_outputs'] = list()

                                    aoutput = {'module_name':module_name, 'module_value':module_value, 'module_type':mtype}
                                    if os.path.isdir(os.path.join(outputdir, 'analyzer_output', d, dd)):
                                        aoutput['data_type'] = 'dir'
                                    else:
                                        aoutput['data_type'] = 'file'
                                    results[script]['analyzer_outputs'].append(aoutput)

                    analyzer_status[script] = {}
                    analyzer_status[script].update(results[script])
                else:
                    self._logger.debug("skipping analyzer (no change in analyzer/config and prior run succeeded): " + script)

        # process and store analyzer outputs
        didsave = False
        for script in results.keys():
            result = results[script]
            if result['status'] == 'SUCCESS':
                mtype = None
                if result['atype'] == 'user_scripts_dir':
                    mtype = 'user'
                elif result['atype'] == 'extra_scripts_dir':
                    mtype = 'extra'

                if os.path.exists(os.path.join(result['outputdir'], 'analyzer_output')):
                    for d in os.listdir(os.path.join(result['outputdir'], 'analyzer_output')):
                        if os.path.exists(os.path.join(result['outputdir'], 'analyzer_output', d)):
                            for dd in os.listdir(os.path.join(result['outputdir'], 'analyzer_output', d)):
                                dfile = os.path.join(result['outputdir'], 'analyzer_output', d, dd)
                                module_name = d
                                module_value = dd
                                if os.path.isfile(dfile):
                                    adata = anchore_utils.read_kvfile_todict(dfile)
                                    self.anchoreDB.save_analysis_output(image.meta['imageId'], module_name, module_value, adata, module_type=mtype)
                                    didsave = True
                                elif os.path.isdir(dfile):
                                    self.anchoreDB.save_analysis_output(image.meta['imageId'], module_name, module_value, dfile, module_type=mtype, directory_data=True)
                                    didsave = True

        self.anchoreDB.save_analyzer_manifest(image.meta['imageId'], analyzer_status)

        if success:
            self._logger.debug("analyzer commands all finished with successful exit codes")

            if didsave:
                self._logger.debug("generating analysis report from analyzer outputs and saving")                
                report = self.generate_analysis_report(image)
                self.anchoreDB.save_analysis_report(image.meta['imageId'], report)

            self._logger.debug("saving image information with updated analysis data")
            image.save_image()

            self._logger.info(image.meta['shortId'] + ": analyzed.")


        self._logger.debug("running analyzers on image: " + str(image.meta['imagename']) + ": end")

        return(success)

    def generate_analysis_report(self, image):
        # this routine reads the results of image analysis and generates a formatted report
        report = {}
        amanifest = self.anchoreDB.load_analyzer_manifest(image.meta['imageId'])
        for amodule in amanifest.keys():
            if 'analyzer_outputs' in amanifest[amodule]:
                for aoutput in amanifest[amodule]['analyzer_outputs']:
                    module_name = aoutput['module_name']
                    module_value = aoutput['module_value']
                    module_type = aoutput['module_type']
                    data_type = aoutput['data_type']
                    if module_name not in report:
                        report[module_name] = {}
                    if module_value not in report[module_name]:
                        report[module_name][module_value] = {}

                    if data_type == 'file':
                        adata = self.anchoreDB.load_analysis_output(image.meta['imageId'], module_name, module_value, module_type=module_type)
                    else:
                        adata = {}

                    report[module_name][module_value][module_type] = adata
        return(report)

    def run(self):
        self._logger.debug("main image analysis on images: " + str(self.images) + ": begin")
        # analyze image and all of its family members
        success = True
        toanalyze = {}
        comparehash = {}
        linkhash = {}

        # calculate all images to be analyzed
        for imageId in self.images:
            coreimage = self.allimages[imageId]

            toanalyze[coreimage.meta['imageId']] = coreimage

            base = False
            lastimage = coreimage
            for i in coreimage.anchore_familytree:
                image = self.allimages[i]
                toanalyze[image.meta['imageId']] = image

                if (image.meta['shortId'] != coreimage.meta['shortId'] and not image.is_intermediate()):
                    comparehash[coreimage.meta['shortId'] + image.meta['shortId']] = [coreimage, image]
                    comparehash[lastimage.meta['shortId'] + image.meta['shortId']] = [lastimage, image]
                    if not base and image.is_base():
                        base = image
                    lastimage = image

            if base:
                linkhash[image.meta['imageId']] = base.meta['imageId']

        # execute analyzers
        self._logger.debug("images to be analyzed: " + str(toanalyze.keys()))
        for imageId in toanalyze.keys():
            image = toanalyze[imageId]
            success = self.run_analyzers(image)
            if not success:
                self._logger.error("analyzer failed to run on image " + str(image.meta['imagename']) + ", skipping the rest")
                break

        if not success:
            self._logger.error("analyzers failed to run on one or more images.")
            return (False)

        if not self.skipgates:
            # execute gates
            self._logger.debug("running gates post-analysis: begin")
            for imageId in toanalyze.keys():
                c = controller.Controller(anchore_config=self.config, imagelist=[imageId], allimages=self.allimages).run_gates(refresh=True)
            self._logger.debug("running gates post-analysis: end")

        self._logger.debug("main image analysis on images: " + str(self.images) + ": end")
        return (success)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import rpm
import subprocess
import stat

import anchore.anchore_utils

def rpm_check_file_membership_from_path(unpackdir, allfiles=None):
    rpmfiles = {}
    matchfiles = list()
    nonmatchfiles = list()
    realnonmatchfiles = list()
    inpath = os.path.join(unpackdir, 'rootfs')
    rpmdbdir = anchore.anchore_utils.rpm_prepdb(unpackdir)

    if not allfiles:
        filemap, allfiles = anchore.anchore_utils.get_files_from_path(inpath)

    try:
        try:
            # get a list of all files from RPM
            try:
                sout = subprocess.check_output(['rpm', '--dbpath='+rpmdbdir, '-qal'])
                sout = sout.decode('utf8')
            except subprocess.CalledProcessError as err:
                sout = ""
                errmsg = err.output.decode('utf8')

            for l in sout.splitlines():
                l = l.strip()
                rpmfiles[l] = True
        except Exception as err:
            raise err

        # find any rpm files that are not in the filesystem (first past)
        for rfile in allfiles.keys():
            if rfile not in rpmfiles:
                nonmatchfiles.append(rfile)

        # second pass - hardlinks make this necessary
        done=False
        start = 0
        while not done:
            cmdlist = nonmatchfiles[start:start+256]
            if len(cmdlist) <= 0:
                done=True
            else:
                try:
                    sout = subprocess.check_output(['rpm', '--dbpath='+rpmdbdir, '-qf'] + cmdlist, stderr=subprocess.STDOUT)
                    sout = sout.decode('utf8')
                except subprocess.CalledProcessError as err:
                    sout = err.output.decode('utf8')

                for l in sout.splitlines():
                    l = l.strip()
                    try:
                        filename = re.match("file (.*) is not owned by any package", l).group(1)
                        realnonmatchfiles.append(filename)
                    except:
                        pass
            start = start + 256
    except Exception as err:
        raise err
    
    # for all files, if not unmatched, consider them matched to a package
    for rfile in allfiles.keys():
        if rfile not in realnonmatchfiles:
            matchfiles.append(rfile)

    #print "RESULT: " + str(len(matchfiles)) + " : " + str(len(realnonmatchfiles))
    return(matchfiles, realnonmatchfiles)

def dpkg_check_file_membership_from_path(unpackdir, allfiles=None):
    matchfiles = list()
    nonmatchfiles = list()
    inpath = os.path.join(unpackdir, 'rootfs')

    if not allfiles:
        filemap, allfiles = anchore.anchore_utils.get_files_from_path(inpath)

    try:
        try:

            for flist in anchore.anchore_utils.grouper(allfiles.keys(), 256):
                try:
                    sout = subprocess.check_output(['dpkg', "--admindir="+unpackdir+"/rootfs/var/lib/dpkg", '-S'] + flist, stderr=subprocess.STDOUT)
                    sout = sout.decode('utf8')
                except subprocess.CalledProcessError as err:
                    sout = err.output.decode('utf8')

                for l in sout.splitlines():
                    l = l.strip()
                    try:
                        filename = re.match("dpkg-query: no path found matching pattern (.*)", l).group(1)
                        nonmatchfiles.append(filename)
                    except:
                        pass

        except Exception as err:
            print str(err)

    except Exception as err:
        raise err

    matchfiles = list(set(allfiles.keys()) - set(nonmatchfiles))

    #print "RESULT: " + str(len(matchfiles)) + " : " + str(len(nonmatchfiles))

    return(matchfiles, nonmatchfiles)

analyzer_name = "file_list"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
imgid = config['imgid_full']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

meta = anchore.anchore_utils.get_distro_from_path('/'.join([unpackdir, "rootfs"]))
distrodict = anchore.anchore_utils.get_distro_flavor(meta['DISTRO'], meta['DISTROVERS'], meta['LIKEDISTRO'])

simplefiles = {}
outfiles = {}
nonpkgoutfiles = {}

try:
    allfiles = {}
    if os.path.exists(unpackdir + "/anchore_allfiles.json"):
        with open(unpackdir + "/anchore_allfiles.json", 'r') as FH:
            allfiles = json.loads(FH.read())
    else:
        fmap, allfiles = anchore.anchore_utils.get_files_from_path(unpackdir + "/rootfs")
        with open(unpackdir + "/anchore_allfiles.json", 'w') as OFH:
            OFH.write(json.dumps(allfiles))

    # fileinfo
    for name in allfiles.keys():
        outfiles[name] = json.dumps(allfiles[name])
        simplefiles[name] = oct(stat.S_IMODE(allfiles[name]['mode']))

    if distrodict['flavor'] == "RHEL":
        # rpm file check
        match, nonmatch = rpm_check_file_membership_from_path(unpackdir, allfiles=allfiles)
        for f in nonmatch:
            nonpkgoutfiles[f] = 'NOTPKGED'
    elif distrodict['flavor'] == "DEB":
        # dpkg file check
        match, nonmatch = dpkg_check_file_membership_from_path(unpackdir, allfiles=allfiles)
        for f in nonmatch:
            nonpkgoutfiles[f] = 'NOTPKGED'

except Exception as err:
    import traceback
    traceback.print_exc()
    raise err

if simplefiles:
    ofile = os.path.join(outputdir, 'files.all')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, simplefiles)

if outfiles:
    ofile = os.path.join(outputdir, 'files.allinfo')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, outfiles)
if nonpkgoutfiles:
    ofile = os.path.join(outputdir, 'files.nonpkged')
    anchore.anchore_utils.write_kvfile_fromdict(ofile, nonpkgoutfiles)

sys.exit(0)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import subprocess
import anchore.anchore_utils

analyzer_name = "analyzer_meta"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

try:
    meta = anchore.anchore_utils.get_distro_from_path(os.path.join(unpackdir, "rootfs"))

    dockerfile_contents = None
    if os.path.exists(os.path.join(unpackdir, "Dockerfile")):
        dockerfile_contents = anchore.anchore_utils.read_plainfile_tostr(os.path.join(unpackdir, "Dockerfile"))

    if meta:
        ofile = os.path.join(outputdir, 'analyzer_meta')
        anchore.anchore_utils.write_kvfile_fromdict(ofile, meta)
        shutil.copy(ofile, unpackdir + "/analyzer_meta")
    else:
        raise Exception("could not analyze/store basic metadata about image")

    if dockerfile_contents:
        ofile = os.path.join(outputdir, 'Dockerfile')
        anchore.anchore_utils.write_plainfile_fromstr(ofile, dockerfile_contents)

except Exception as err:
    raise err

sys.exit(0)





#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

def perform_cve_scan_simple(severity='all'):
    ret = {}
    warns = list()

    pkgmaps = {}
    imgmaps = {}
    flvmaps = {}

    cve_data_context = {}
    for imageId in config['images']:

        distrometa = anchore.anchore_utils.get_distro_from_imageId(imageId)
        idistro = distrometa['DISTRO']
        idistrovers = distrometa['DISTROVERS']
        distrodict = anchore.anchore_utils.get_distro_flavor(idistro, idistrovers)
        flavor = distrodict['flavor']

        try:
            dstr, new_cve_data = anchore.anchore_utils.cve_load_data_imageId(imageId, cve_data_context=cve_data_context)

            if dstr not in pkgmaps:
                pkgmaps[dstr] = {}

            if dstr not in imgmaps:
                imgmaps[dstr] = list()
                
            if dstr not in flvmaps:
                flvmaps[dstr] = flavor

            imgmaps[dstr].append(imageId)

            pkgmap = pkgmaps[dstr]

            pkgs = anchore.anchore_utils.load_analysis_output(imageId, 'package_list', 'pkgs.all')
            pkgs.update(anchore.anchore_utils.load_analysis_output(imageId, 'package_list', 'pkgs_plus_source.all'))

            for pkg in pkgs.keys():
                vers = pkgs[pkg]

                if pkg not in pkgmap:
                    pkgmap[pkg] = {'versions':{}}

                if vers not in pkgmap[pkg]['versions']:
                    pkgmap[pkg]['versions'][vers] = list()

                pkgmap[pkg]['versions'][vers].append(imageId)

        except Exception as err:
            warns.append(config['meta']['shortId']+" ("+ config['meta']['humanname'] +") cannot scan image: no CVE data available for distro ("+str(idistro)+":"+str(idistrovers)+")")

    for dstr in pkgmaps.keys():
        cve_results = anchore.anchore_utils.cve_scanimages(imgmaps[dstr], pkgmaps[dstr], flvmaps[dstr], cve_data_context[dstr])

        if cve_results:
            for cveId in cve_results.keys():
                if severity == 'all' or cve_results[cveId]['severity'] == severity:
                    if cveId not in ret:
                        ret[cveId] = {'severity':'Unknown', 'imageIds':list()}

                    outsev = "N/A"
                    if 'severity' in cve_results[cveId]:
                        outsev = cve_results[cveId]['severity']

                    outurl = "N/A"
                    if 'url' in cve_results[cveId]:
                        outurl = cve_results[cveId]['url']

                    ret[cveId]['severity'] = outsev
                    ret[cveId]['url'] = outurl
                    ret[cveId]['imageIds'] = ret[cveId]['imageIds'] + cve_results[cveId]['images']
        
    return(ret, warns)

# main routine

# parse cmdline into config dict
try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <Low|Medium|High|Critical|Unknown|Negligible> <another level> ...\nhelp: use 'all' to scan for all severity levels")
except Exception as err:
    print str(err)
    sys.exit(1)

# this means that no config is available, for example when 'help' is given as a parameter
if not config:
    sys.exit(0)

# handle default case if no params are given
if len(config['params']) <= 0:
    config['params'] = ['all']

# set up output list
outlist = list()
outlist.append(["*CVE_ID","Severity","Affected_Count", "Affected_Image_Ids", "URL"])

# perform query
result = {}
warns = list()

try:
    result, warns = perform_cve_scan_simple('all')
except Exception as err:
    warns.append("could not perform CVE scan due to unexpected error: " + str(err))

for k in result.keys():
    if 'all' in config['params'] or result[k]['severity'] in config['params']:
        outlist.append([k, result[k]['severity'], str(len(result[k]['imageIds'])), ' '.join(result[k]['imageIds']), result[k]['url']])

# save output list as output file for anchore to ingress
anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)
if len(warns) > 0:
    anchore.anchore_utils.write_plainfile_fromlist(config['output_warns'], warns)

# cleanup and exit
sys.exit(0)
